#!/usr/bin/env python

# external
import numpy as np
from sympy import symbols
from numpy.testing import assert_allclose
from nose.tools import assert_raises
try:
    import pythreejs as p3js
except ImportError:
    print('pythreejs not installed, skipping related tests.')
    p3js = None

# local
from ..shapes import (Shape, Box, Cube, Cylinder, Cone, Sphere, Circle, Plane,
                      Tetrahedron, Octahedron, Icosahedron, Torus, Tube,
                      TorusKnot)


def test_shape():

    shape = Shape(name='shape', color='blue', material="DIRT")

    assert shape.name == 'shape'
    assert shape.__str__() == 'Shape shape color:blue material:DIRT'
    assert shape.__repr__() == 'Shape'
    assert shape.color == 'blue'
    assert shape.material == "DIRT"

    shape.name = 'shape1'
    assert shape.name == 'shape1'
    with assert_raises(TypeError):
        shape.name = 1

    shape.color = 'red'
    assert shape.color == 'red'
    with assert_raises(ValueError):
        shape.color = 'puke'

    shape.material = "water"
    assert shape.material == "water"
    shape.material = "WATER"
    assert shape.material == "WATER"
    with assert_raises(ValueError):
        shape.material = 'fluffy cloth'

    assert shape.generate_dict() == {"color": "red",
                                     "type": "Shape",
                                     "name": "shape1",
                                     "material": "WATER"}

    assert isinstance(shape, Shape)

    # testing unnamed
    shape_ = Shape(color='blue')

    assert shape_.name == 'unnamed'
    assert shape_.__str__() == 'Shape unnamed color:blue material:default'
    assert shape_.__repr__() == 'Shape'


def test_shape_geometry_with_expressions():

    shape = Shape()
    shape.length = symbols('l')
    shape.geometry_attrs.append('length')
    expected = {"color": "grey",
                "type": "Shape",
                "name": "unnamed",
                "length": 16.0,
                "material": "default"}
    actual = shape.generate_dict(constant_map={symbols('l'): 16.0})
    assert actual == expected

    shape = Shape()
    shape.length = symbols('l1') + symbols('l2') ** 2
    shape.geometry_attrs.append('length')
    expected = {"color": "grey",
                "type": "Shape",
                "name": "unnamed",
                "length": 20.0,
                "material": "default"}
    actual = shape.generate_dict(constant_map={symbols('l1'): 4.0,
                                               symbols('l2'): 4.0})
    assert actual == expected


def test_box():

    box = Box(10.0, 20.0, 30.0, name='box', color='blue', material="WATER")

    assert box.name == 'box'
    assert box.__str__() == 'Box box color:blue material:WATER depth:30.0 height:20.0 width:10.0'
    assert box.__repr__() == 'Box'
    assert box.width == 10.0
    assert box.height == 20.0
    assert box.depth == 30.0
    assert box.color == 'blue'

    if p3js is not None:
        mesh = box._p3js_mesh()
        expected_mesh = p3js.Mesh(p3js.BoxBufferGeometry(width=10.0,
                                                         height=20.0,
                                                         depth=30.0),
                                  p3js.MeshStandardMaterial(color='blue'),
                                  name='box')
        assert repr(mesh) == repr(expected_mesh)


def test_cube():

    cube = Cube(10.0, name='cube', color='blue', material="WATER")

    assert cube.name == 'cube'
    assert cube.__str__() == 'Cube cube color:blue material:WATER length:10.0'
    assert cube.__repr__() == 'Cube'
    assert cube.length == 10.0
    assert cube.color == 'blue'

    if p3js is not None:
        mesh = cube._p3js_mesh()
        expected_mesh = p3js.Mesh(p3js.BoxBufferGeometry(width=10.0,
                                                         height=10.0,
                                                         depth=10.0),
                                  p3js.MeshStandardMaterial(color='blue'),
                                  name='cube')
        assert repr(mesh) == repr(expected_mesh)

    cube.name = 'cube1'
    assert cube.name == 'cube1'

    cube.length = 16.0
    assert cube.length == 16.0

    cube.color = 'red'
    assert cube.color == 'red'

    assert cube.generate_dict() == {"color": "red",
                                    "type": "Cube",
                                    "name": "cube1",
                                    "length": 16.0,
                                    "material": "WATER"}

    assert isinstance(cube, Shape)

    if p3js is not None:
        mesh = cube._p3js_mesh()
        expected_mesh = p3js.Mesh(p3js.BoxBufferGeometry(width=16.0,
                                                         height=16.0,
                                                         depth=16.0),
                                  p3js.MeshStandardMaterial(color='red'),
                                  name='cube1')
        assert repr(mesh) == repr(expected_mesh)

    # testing unnamed
    cube = Cube(10.0, color='blue')

    assert cube.name == 'unnamed'
    expected = 'Cube unnamed color:blue material:default length:10.0'
    assert cube.__str__() == expected
    assert cube.__repr__() == 'Cube'

    cube = Cube(symbols('V') ** (1.0 / 3.0))
    actual = cube.generate_dict(constant_map={symbols('V'): 27.0})
    assert actual == {"color": "grey",
                      "type": "Cube",
                      "name": "unnamed",
                      "length": 3.0,
                      "material": "default"}

    if p3js is not None:
        mesh = cube._p3js_mesh(constant_map={symbols('V'): 27.0})
        expected_mesh = p3js.Mesh(p3js.BoxBufferGeometry(width=3.0, height=3.0,
                                                         depth=3.0),
                                  p3js.MeshStandardMaterial(color='gray'),
                                  name='unnamed')
        assert repr(mesh) == repr(expected_mesh)


def test_cylinder():

    cylinder = Cylinder(10.0, 5.0, name='cylinder', color='blue',
                        material="METAL")
    assert cylinder.name == 'cylinder'
    assert cylinder.__str__() == \
        'Cylinder cylinder color:blue material:METAL length:10.0 radius:5.0'
    assert cylinder.__repr__() == 'Cylinder'
    assert cylinder.length == 10.0
    assert cylinder.radius == 5.0
    assert cylinder.color == 'blue'

    if p3js is not None:
        mesh = cylinder._p3js_mesh()
        expected_mesh = p3js.Mesh(p3js.CylinderBufferGeometry(radiusTop=5.0,
                                                              radiusBottom=5.0,
                                                              height=10.0,
                                                              radialSegments=100),
                                  p3js.MeshStandardMaterial(color='blue'),
                                  name='cylinder')
        assert repr(mesh) == repr(expected_mesh)

    cylinder.name = 'cylinder1'
    assert cylinder.name == 'cylinder1'

    cylinder.length = 14.0
    assert cylinder.length == 14.0

    cylinder.radius = 7.0
    assert cylinder.radius == 7.0

    cylinder.color = 'cyan'
    assert cylinder.color == 'cyan'

    cylinder.material = 'FOIL'
    assert cylinder.material == 'FOIL'
    assert cylinder.generate_dict() == {"color": "cyan",
                                        "type": "Cylinder",
                                        "name": "cylinder1",
                                        "length": 14.0,
                                        "radius": 7.0,
                                        "material": "FOIL"}

    assert isinstance(cylinder, Shape)

    cylinder_ = Cylinder(10.0, 5.0, color='blue')
    assert cylinder_.name == 'unnamed'
    assert cylinder_.__str__() == \
        'Cylinder unnamed color:blue material:default length:10.0 radius:5.0'

    assert cylinder_.__repr__() == 'Cylinder'


def test_cone():

    cone = Cone(10.0, 5.0, name='cone', color='darkblue',
                material="CHECKERBOARD")
    assert cone.name == 'cone'
    assert cone.__str__() == \
        'Cone cone color:darkblue material:CHECKERBOARD length:10.0 radius:5.0'
    assert cone.__repr__() == 'Cone'
    assert cone.length == 10.0
    assert cone.radius == 5.0
    assert cone.color == 'darkblue'

    if p3js is not None:
        mesh = cone._p3js_mesh()
        expected_mesh = p3js.Mesh(p3js.CylinderBufferGeometry(radiusTop=0.0,
                                                              radiusBottom=5.0,
                                                              height=10.0),
                                  p3js.MeshStandardMaterial(color='darkblue'),
                                  name='cone')
        assert repr(mesh) == repr(expected_mesh)

    cone.name = 'cone1'
    assert cone.name == 'cone1'

    cone.length = 16.0
    assert cone.length == 16.0

    cone.radius = 3.0
    assert cone.radius == 3.0

    cone.color = 'darkcyan'
    assert cone.color == 'darkcyan'

    assert cone.generate_dict() == {"color": "darkcyan",
                                    "type": "Cone",
                                    "name": "cone1",
                                    "length": 16.0,
                                    "radius": 3.0,
                                    "material": "CHECKERBOARD"}
    assert isinstance(cone, Shape)

    cone_ = Cone(10.0, 5.0, color='darkblue')
    assert cone_.name == 'unnamed'
    assert cone_.__str__() == \
        'Cone unnamed color:darkblue material:default length:10.0 radius:5.0'
    assert cone_.__repr__() == 'Cone'


def test_sphere():

    sphere = Sphere(10.0, name='sphere', color='azure')

    assert sphere.name == 'sphere'
    assert sphere.__str__() == \
        'Sphere sphere color:azure material:default radius:10.0'
    assert sphere.__repr__() == 'Sphere'
    assert sphere.radius == 10.0
    assert sphere.color == 'azure'

    if p3js is not None:
        mesh = sphere._p3js_mesh()
        expected_mesh = p3js.Mesh(p3js.SphereBufferGeometry(radius=10.0,
                                                            widthSegments=100,
                                                            heightSegments=100),
                                  p3js.MeshStandardMaterial(color='azure'),
                                  name='sphere')
        assert repr(mesh) == repr(expected_mesh)

    sphere.name = 'sphere1'
    assert sphere.name == 'sphere1'

    sphere.radius = 14.0
    assert sphere.radius == 14.0

    sphere.color = 'aqua'
    assert sphere.color == 'aqua'

    assert sphere.generate_dict() == {"color":  "aqua",
                                      "type": "Sphere",
                                      "name": "sphere1",
                                      "radius": 14.0,
                                      "material": "default"}
    assert isinstance(sphere, Shape)

    sphere_ = Sphere(10.0, color='azure')
    assert sphere_.name == 'unnamed'
    assert sphere_.__str__() == \
        'Sphere unnamed color:azure material:default radius:10.0'
    assert sphere_.__repr__() == 'Sphere'


def test_circle():

    circle = Circle(10.0, name='circle', color='gold')

    assert circle.name == 'circle'
    assert circle.__str__() == \
        'Circle circle color:gold material:default radius:10.0'
    assert circle.__repr__() == 'Circle'
    assert circle.radius == 10.0
    assert circle.color == 'gold'

    if p3js is not None:
        mesh = circle._p3js_mesh()
        expected_mesh = p3js.Mesh(p3js.CircleBufferGeometry(radius=10.0,
                                                            segments=100),
                                  p3js.MeshStandardMaterial(color='gold'),
                                  name='circle')
        assert repr(mesh) == repr(expected_mesh)

    circle.name = 'circle1'
    assert circle.name == 'circle1'

    circle.radius = 12.0
    assert circle.radius == 12.0

    circle.color = 'black'
    assert circle.color == 'black'

    assert circle.generate_dict() == {"color": "black",
                                      "type": "Circle",
                                      "name": "circle1",
                                      "radius": 12.0,
                                      "material": "default"}

    assert isinstance(circle, Shape)

    circle = Circle(10.0, color='gold')
    assert circle.name == 'unnamed'
    assert circle.__str__() == \
        'Circle unnamed color:gold material:default radius:10.0'
    assert circle.__repr__() == 'Circle'


def test_plane():

    plane = Plane(10.0, 20.0, name='plane', color='lightcyan')
    assert plane.name == 'plane'
    assert plane.__str__() == \
        'Plane plane color:lightcyan material:default length:10.0 width:20.0'
    assert plane.__repr__() == 'Plane'
    assert plane.length == 10.0
    assert plane.width == 20.0
    assert plane.color == 'lightcyan'

    if p3js is not None:
        mesh = plane._p3js_mesh()
        expected_mesh = p3js.Mesh(p3js.PlaneBufferGeometry(height=10.0,
                                                           width=20.0),
                                  p3js.MeshStandardMaterial(color='lightcyan',
                                                            side='DoubleSide'),
                                  name='plane')
        assert repr(mesh) == repr(expected_mesh)

    plane.name = 'plane1'
    assert plane.name == 'plane1'

    plane.length = 30.0
    assert plane.length == 30.0

    plane.width = 10.0
    assert plane.width == 10.0

    plane.color = 'lavender'
    assert plane.color == 'lavender'

    assert plane.generate_dict() == {"color":  "lavender",
                                     "type": "Plane",
                                     "name": "plane1",
                                     "width": 10.0,
                                     "length": 30.0,
                                     "material": "default"}

    assert isinstance(plane, Shape)

    plane_ = Plane(10.0, 20.0, color='indigo')
    assert plane_.name == 'unnamed'
    assert plane_.__str__() == \
        'Plane unnamed color:indigo material:default length:10.0 width:20.0'
    assert plane_.__repr__() == 'Plane'


def test_tetrahedron():
    # Tetrahedron, Octahedron and Icosahedron geometry is defined by the
    # radius of the circumscribed sphere. It would be mentioned explicitly
    # in the docstrings
    tetrahedron = Tetrahedron(5.0, name='tetrahedron', color='maroon')
    assert tetrahedron.name == 'tetrahedron'
    assert tetrahedron.__str__() == \
        'Tetrahedron tetrahedron color:maroon material:default radius:5.0'
    assert tetrahedron.__repr__() == 'Tetrahedron'
    assert tetrahedron.radius == 5.0
    assert tetrahedron.color == 'maroon'

    if p3js is not None:
        mesh = tetrahedron._p3js_mesh()
        expected_mesh = p3js.Mesh(p3js.TetrahedronGeometry(radius=5.0),
                                  p3js.MeshStandardMaterial(color='maroon'),
                                  name='tetrahedron')
        assert repr(mesh) == repr(expected_mesh)

    tetrahedron.name = 'tetrahedron1'
    assert tetrahedron.name == 'tetrahedron1'

    tetrahedron.radius = 7.0
    assert tetrahedron.radius == 7.0

    tetrahedron.color = 'orange'
    assert tetrahedron.color == 'orange'

    assert tetrahedron.generate_dict() == {"color":  "orange",
                                           "type": "Tetrahedron",
                                           "name": "tetrahedron1",
                                           "radius": 7.0,
                                           "material": "default"}
    assert isinstance(tetrahedron, Shape)

    tetrahedron_ = Tetrahedron(5.0, color='maroon')
    assert tetrahedron_.name == 'unnamed'
    assert tetrahedron_.__str__() == \
        'Tetrahedron unnamed color:maroon material:default radius:5.0'
    assert tetrahedron_.__repr__() == 'Tetrahedron'


def test_octahedron():

    octahedron = Octahedron(12.0, name='octahedron', color='purple')
    assert octahedron.name == 'octahedron'
    assert octahedron.__str__() == \
        'Octahedron octahedron color:purple material:default radius:12.0'
    assert octahedron.__repr__() == 'Octahedron'
    assert octahedron.radius == 12.0
    assert octahedron.color == 'purple'

    if p3js is not None:
        mesh = octahedron._p3js_mesh()
        expected_mesh = p3js.Mesh(p3js.OctahedronGeometry(radius=12.0),
                                  p3js.MeshStandardMaterial(color='purple'),
                                  name='octahedron')
        assert repr(mesh) == repr(expected_mesh)

    octahedron.name = 'octahedron1'
    assert octahedron.name == 'octahedron1'

    octahedron.radius = 2.0
    assert octahedron.radius == 2.0

    octahedron.color = 'red'
    assert octahedron.color == 'red'

    assert octahedron.generate_dict() == {"color": "red",
                                          "type": "Octahedron",
                                          "name": "octahedron1",
                                          "radius": 2.0,
                                          "material": "default"}

    assert isinstance(octahedron, Shape)

    octahedron_ = Octahedron(12.0, color='purple')
    assert octahedron_.name == 'unnamed'
    assert octahedron_.__str__() == \
        'Octahedron unnamed color:purple material:default radius:12.0'
    assert octahedron_.__repr__() == 'Octahedron'


def test_icosahedron():

    icosahedron = Icosahedron(11.0, name='icosahedron', color='blue')
    assert icosahedron.name == 'icosahedron'
    assert icosahedron.__str__() == \
        'Icosahedron icosahedron color:blue material:default radius:11.0'
    assert icosahedron.__repr__() == 'Icosahedron'
    assert icosahedron.radius == 11.0
    assert icosahedron.color == 'blue'

    if p3js is not None:
        mesh = icosahedron._p3js_mesh()
        expected_mesh = p3js.Mesh(p3js.IcosahedronGeometry(radius=11.0),
                                  p3js.MeshStandardMaterial(color='blue'),
                                  name='icosahedron')
        assert repr(mesh) == repr(expected_mesh)

    icosahedron.name = 'icosahedron1'
    assert icosahedron.name == 'icosahedron1'

    icosahedron.radius = 3.0
    assert icosahedron.radius == 3.0

    icosahedron.color = 'blue'
    assert icosahedron.color == 'blue'

    assert icosahedron.generate_dict() == {"color": "blue",
                                           "type": "Icosahedron",
                                           "name": "icosahedron1",
                                           "radius": 3.0,
                                           "material": "default"}

    assert isinstance(icosahedron, Shape)

    icosahedron_ = Icosahedron(11.0, color='blue')
    assert icosahedron_.name == 'unnamed'
    assert icosahedron_.__str__() == \
        'Icosahedron unnamed color:blue material:default radius:11.0'
    assert icosahedron_.__repr__() == 'Icosahedron'


def test_torus():
    torus = Torus(10.0, 2.0, name='torus', color='red')

    assert torus.name == 'torus'
    assert torus.__str__() == \
        'Torus torus color:red material:default radius:10.0 tube_radius:2.0'
    assert torus.__repr__() == 'Torus'
    assert torus.radius == 10.0
    assert torus.tube_radius == 2.0
    assert torus.color == 'red'

    if p3js is not None:
        mesh = torus._p3js_mesh()
        expected_mesh = p3js.Mesh(p3js.TorusBufferGeometry(radius=10.0,
                                                           tube=2.0),
                                  p3js.MeshStandardMaterial(color='red'),
                                  name='torus')
        assert repr(mesh) == repr(expected_mesh)

    torus.name = 'torus1'
    assert torus.name == 'torus1'

    torus.radius = 15.0
    assert torus.radius == 15.0

    torus.tube_radius = 4.0
    assert torus.tube_radius == 4.0

    torus.color = 'tan'
    assert torus.color == 'tan'

    assert torus.generate_dict() == {"color": "tan",
                                     "type": "Torus",
                                     "name": "torus1",
                                     "radius": 15.0,
                                     "tube_radius": 4.0,
                                     "material": "default"}

    assert isinstance(torus, Shape)

    torus_ = Torus(10.0, 2.0, color='red')
    assert torus_.name == 'unnamed'
    assert torus_.__str__() == \
        'Torus unnamed color:red material:default radius:10.0 tube_radius:2.0'
    assert torus_.__repr__() == 'Torus'


def test_tube():

    point_list = [[2., 4., 5.], [2., 6., 4.], [1., 5., 8.]]

    tube = Tube(10.0, point_list, name='tube', color='red')

    assert tube.name == 'tube'
    assert tube.__str__() == ('Tube tube color:red material:default points:'
                              '[[2.0, 4.0, 5.0], [2.0, 6.0, 4.0], '
                              '[1.0, 5.0, 8.0]] radius:10.0')
    assert tube.__repr__() == 'Tube'
    assert tube.radius == 10.0
    assert_allclose(tube.points, point_list)
    assert tube.color == 'red'

    tube.name = 'tube1'
    assert tube.name == 'tube1'

    tube.radius = 15.0
    assert tube.radius == 15.0

    new_point_list = [[3., 4., 5.], [1, 6., 8.], [2., 7., 3.]]
    tube.points = new_point_list
    assert_allclose(tube.points, new_point_list)

    tube.color = 'pink'
    assert tube.color == 'pink'

    actual = tube.generate_dict()

    expected = {"color": "pink",
                "type": "Tube",
                "name": "tube1",
                "radius": 15.0,
                "points": np.asarray(new_point_list),
                "material": "default"}

    for key in ['color', 'type', 'name', 'radius']:
        actual[key] == expected[key]
    assert_allclose(actual['points'], expected['points'])

    assert isinstance(tube, Shape)

    tube_ = Tube(10.0, point_list, color='red')
    assert tube_.name == 'unnamed'
    assert tube_.__str__() == ('Tube unnamed color:red material:default points:'
                               '[[2.0, 4.0, 5.0], [2.0, 6.0, 4.0], '
                               '[1.0, 5.0, 8.0]] radius:10.0')
    assert tube_.__repr__() == 'Tube'


def test_torus_knot():

    torus_knot = TorusKnot(10.0, 2.0, name='torus_knot', color='red')

    assert torus_knot.name == 'torus_knot'
    assert torus_knot.__str__() == ('TorusKnot torus_knot color:red '
                                    'material:default radius:10.0 '
                                    'tube_radius:2.0')
    assert torus_knot.__repr__() == 'TorusKnot'
    assert torus_knot.radius == 10.0
    assert torus_knot.tube_radius == 2.0
    assert torus_knot.color == 'red'

    if p3js is not None:
        mesh = torus_knot._p3js_mesh()
        expected_mesh = p3js.Mesh(p3js.TorusKnotBufferGeometry(radius=10.0,
                                                               tube=2.0),
                                  p3js.MeshStandardMaterial(color='red'),
                                  name='torus_knot')
        assert repr(mesh) == repr(expected_mesh)

    torus_knot.name = 'torus_knot1'
    assert torus_knot.name == 'torus_knot1'

    torus_knot.radius = 12.0
    assert torus_knot.radius == 12.0

    torus_knot.tube_radius = 1.0
    assert torus_knot.tube_radius == 1.0

    torus_knot.color = 'blue'
    assert torus_knot.color == 'blue'

    assert torus_knot.generate_dict() == {"color": "blue",
                                          "type": "TorusKnot",
                                          "name": "torus_knot1",
                                          "radius": 12.0,
                                          "tube_radius": 1,
                                          "material": "default"}

    assert isinstance(torus_knot, Shape)

    torus_knot_ = TorusKnot(10.0, 2.0, color='red')
    assert torus_knot_.name == 'unnamed'
    assert torus_knot_.__str__() == ('TorusKnot unnamed color:red material:'
                                     'default radius:10.0 tube_radius:2.0')
    assert torus_knot_.__repr__() == 'TorusKnot'



#!/usr/bin/env python

import os
import shutil
import glob

import numpy as np
from numpy.testing import assert_allclose
from sympy import symbols
import sympy.physics.mechanics as me
from nose.tools import assert_raises

from ...system import System
from ..shapes import Sphere
from ..visualization_frame import VisualizationFrame
from ..camera import PerspectiveCamera, OrthoGraphicCamera
from ..light import PointLight
from ..scene import Scene
from ...utils import sympy_newer_than


class TestScene(object):

    def setup(self):
        """Setups a simple 1 DoF mass spring damper system visualization."""

        mass, stiffness, damping, gravity = symbols('m, k, c, g')

        position, speed = me.dynamicsymbols('x v')
        positiond = me.dynamicsymbols('x', 1)

        kinematic_equations = [speed - positiond]

        ceiling = me.ReferenceFrame('N')

        origin = me.Point('origin')
        origin.set_vel(ceiling, 0)
        center = origin.locatenew('center', position * ceiling.x)
        center.set_vel(ceiling, speed * ceiling.x)

        block = me.Particle('block', center, mass)
        particles = [block]

        total_force = mass * gravity - stiffness * position - damping * speed
        forces = [(center, total_force * ceiling.x)]

        kane = me.KanesMethod(ceiling, q_ind=[position], u_ind=[speed],
                              kd_eqs=kinematic_equations)

        if sympy_newer_than('1.0'):
            kane.kanes_equations(particles, forces)
        else:
            kane.kanes_equations(forces, particles)


        self.sys = System(kane)
        self.sys.initial_conditions = {position: 0.1, speed: -1.0}
        self.sys.constants = {mass: 1.0, stiffness: 2.0, damping: 3.0,
                              gravity: 9.8}
        self.sys.times = np.linspace(0.0, 0.01, 2)

        sphere = Sphere()

        self.ref_frame = ceiling
        self.origin = origin
        self.viz_frame = VisualizationFrame(ceiling, block, sphere)
        self.viz_frame_sym_shape = VisualizationFrame(ceiling, block,
                                                      Sphere(radius=mass /
                                                             10.0))

    def test_init(self):

        # test minimal args
        scene = Scene(self.ref_frame, self.origin)

        assert scene.reference_frame == self.ref_frame
        assert scene.origin == self.origin
        assert scene.name == 'unnamed'
        # NOTE : There isn't any way to compare the actual camera and light
        # objects because they will be created with different instances of a
        # point. So this is the best bet for now:
        assert scene.cameras[0].name == 'DefaultCamera'
        assert scene.lights[0].name == 'DefaultLight'

        scene.visualization_frames = [self.viz_frame]
        assert scene.visualization_frames == [self.viz_frame]
        assert scene.system is None
        assert scene.times is None
        assert scene.constants is None
        assert scene.states_symbols is None
        assert scene.states_trajectories is None
        assert scene.frames_per_second == 30

        # test maximal args/kwargs
        custom_camera = PerspectiveCamera('my_camera', self.ref_frame,
                                          self.origin.locatenew(
                                              'cam_point', 20 *
                                              self.ref_frame.z))

        custom_light = PointLight('my_light', self.ref_frame,
                                  self.origin.locatenew('light_point', 20 *
                                                        self.ref_frame.y))

        scene = Scene(self.ref_frame, self.origin, self.viz_frame,
                      name='my_scene', cameras=[custom_camera],
                      lights=[custom_light], system=self.sys)

        assert scene.visualization_frames == [self.viz_frame]
        assert scene.name == 'my_scene'
        assert scene.cameras == [custom_camera]
        assert scene.lights == [custom_light]
        assert scene.system is self.sys

        scene = Scene(self.ref_frame, self.origin, self.viz_frame,
                      name='my_scene', cameras=[custom_camera],
                      lights=[custom_light], times=self.sys.times,
                      constants=self.sys.constants,
                      states_symbols=self.sys.states,
                      states_trajectories=self.sys.integrate())

        assert scene.system is None
        assert_allclose(scene.times, self.sys.times)
        assert scene.constants == self.sys.constants
        assert scene.states_symbols == self.sys.states
        assert scene.states_trajectories.shape == (2, 2)

    def test_setting_incompatible_attrs(self):

        scene = Scene(self.ref_frame, self.origin, self.viz_frame,
                      times=self.sys.times)

        with assert_raises(ValueError):
            scene.system = self.sys

        scene = Scene(self.ref_frame, self.origin, self.viz_frame,
                      system=self.sys)

        with assert_raises(ValueError):
            scene.times = self.sys.times

    def test_clear_trajectories(self):
        scene = Scene(self.ref_frame, self.origin, self.viz_frame,
                      times=self.sys.times, constants=self.sys.constants,
                      states_symbols=self.sys.states,
                      states_trajectories=self.sys.integrate())
        scene.clear_trajectories()
        assert scene.system is None
        assert scene.times is None
        assert scene.constants is None
        assert scene.states_symbols is None
        assert scene.states_trajectories is None

    def test_generate_simulation_dict(self):

        scene = Scene(self.ref_frame, self.origin, self.viz_frame,
                      system=self.sys)

        light_id = id(scene.lights[0])
        camera_id = id(scene.cameras[0])
        viz_frame_id = id(scene.visualization_frames[0])

        scene._generate_simulation_dict()

        expected_dict = {viz_frame_id: [[1.0, 0.0, 0.0, 0.0,
                                         0.0, 1.0, 0.0, 0.0,
                                         0.0, 0.0, 1.0, 0.0,
                                         0.1, 0.0, 0.0, 1.0],
                                        [1.0, 0.0, 0.0, 0.0,
                                         0.0, 1.0, 0.0, 0.0,
                                         0.0, 0.0, 1.0, 0.0,
                                         0.09062405469543587, 0.0, 0.0, 1.0]],
                         light_id: [[1.0, 0.0, 0.0, 0.0,
                                     0.0, 1.0, 0.0, 0.0,
                                     0.0, 0.0, 1.0, 0.0,
                                     0.0, 0.0, 10.0, 1.0],
                                    [1.0, 0.0, 0.0, 0.0,
                                     0.0, 1.0, 0.0, 0.0,
                                     0.0, 0.0, 1.0, 0.0,
                                     0.0, 0.0, 10.0, 1.0]],
                         camera_id: [[1.0, 0.0, 0.0, 0.0,
                                      0.0, 1.0, 0.0, 0.0,
                                      0.0, 0.0, 1.0, 0.0,
                                      0.0, 0.0, 10.0, 1.0],
                                     [1.0, 0.0, 0.0, 0.0,
                                      0.0, 1.0, 0.0, 0.0,
                                      0.0, 0.0, 1.0, 0.0,
                                      0.0, 0.0, 10.0, 1.0]]}

        assert (sorted(expected_dict.keys()) ==
                sorted(scene._simulation_info.keys()))

        for k, v in scene._simulation_info.items():
            assert_allclose(v, expected_dict[k])

    def test_generate_scene_dict(self):

        scene = Scene(self.ref_frame, self.origin, self.viz_frame,
                      system=self.sys)

        light_id = id(scene.lights[0])
        camera_id = id(scene.cameras[0])
        viz_frame_id = id(scene.visualization_frames[0])

        # NOTE : generate_simulation_dict must be called before
        # generate_scene_dict
        scene._generate_simulation_dict()
        scene._generate_scene_dict()

        expected_scene_dict = \
            {'newtonian_frame': 'N',
             'name': 'unnamed',
             'workspaceSize': 0.2,
             'source': 'PyDy',
             'lights': {light_id: {'color': 'white',
                                   'init_orientation':
                                       [1.0, 0.0, 0.0, 0.0,
                                        0.0, 1.0, 0.0, 0.0,
                                        0.0, 0.0, 1.0, 0.0,
                                        0.0, 0.0, 10.0, 1.0],
                                   'simulation_id': light_id,
                                   'type': 'PointLight',
                                   'name': 'DefaultLight'}},
             'cameras': {camera_id: {'fov': 45.0,
                                     'name': 'DefaultCamera',
                                     'far': 1000.0,
                                     'simulation_id': camera_id,
                                     'near': 1.0,
                                     'color': 'grey',
                                     'material': 'default',
                                     'reference_frame_name': 'N',
                                     'init_orientation':
                                         [1.0, 0.0, 0.0, 0.0,
                                          0.0, 1.0, 0.0, 0.0,
                                          0.0, 0.0, 1.0, 0.0,
                                          0.0, 0.0, 10.0, 1.0],
                                     'type': 'PerspectiveCamera'}},
             'objects': {viz_frame_id: {'simulation_id': viz_frame_id,
                                        'name': 'unnamed',
                                        'color': 'grey',
                                        'material': 'default',
                                        'reference_frame_name': 'N',
                                        'radius': 10.0,
                                        'init_orientation':
                                            [1.0, 0.0, 0.0, 0.0,
                                             0.0, 1.0, 0.0, 0.0,
                                             0.0, 0.0, 1.0, 0.0,
                                             0.1, 0.0, 0.0, 1.0],
                                        'type': 'Sphere'}}}

        assert scene._scene_info == expected_scene_dict

        # Test the constant_map kwarg.
        scene = Scene(self.ref_frame, self.origin, self.viz_frame_sym_shape,
                      system=self.sys)
        viz_frame_id = id(scene.visualization_frames[0])
        scene._generate_simulation_dict()
        scene._generate_scene_dict()

        assert scene._scene_info['objects'][viz_frame_id]['radius'] == 0.1

    def test_custom_camera(self):

        camera_frame = self.ref_frame.orientnew('rot', 'body',
                                                [np.pi / 2.0,
                                                 np.pi / 2.0,
                                                 np.pi / 2.0], 'xyz')

        camera_point = self.origin.locatenew('cam_point', 30.0 *
                                             camera_frame.z)

        camera = OrthoGraphicCamera('my_camera', camera_frame, camera_point)

        scene = Scene(self.ref_frame, self.origin, self.viz_frame,
                      cameras=[camera], system=self.sys)

        camera_id = id(camera)

        scene._generate_simulation_dict()
        scene._generate_scene_dict()
        scene_dict = scene._scene_info

        expected_orientation_matrix = np.array([0.0, 0.0, 1.0, 0.0,
                                                0.0, -1.0, 0.0, 0.0,
                                                1.0, 0.0, 0.0, 0.0,
                                                30.0, 0.0, 0.0, 1.0])

        assert_allclose(scene_dict['cameras'][camera_id]['init_orientation'],
                        expected_orientation_matrix, atol=1e-14)

        assert scene_dict['cameras'][camera_id]['type'] == 'OrthoGraphicCamera'

    def test_create_static_html(self):

        scene = Scene(self.ref_frame, self.origin, self.viz_frame,
                      system=self.sys)

        # test static dir creation
        scene.create_static_html(overwrite=True, silent=True, prefix='test')
        assert os.path.exists('pydy-resources')
        assert os.path.exists('pydy-resources/index.html')
        assert os.path.exists('pydy-resources/test_scene_desc.json')
        assert os.path.exists('pydy-resources/test_simulation_data.json')

        # test static dir deletion
        scene.remove_static_html(force=True)
        assert not os.path.exists(Scene.pydy_directory)

    def test_generate_visualization_json_system(self):

        scene = Scene(self.ref_frame, self.origin, self.viz_frame)
        scene.generate_visualization_json_system(self.sys)

        # Tests issue #204
        assert scene._scene_info['constant_map'] == {'m': 1.0, 'k': 2.0,
                                                     'c': 3.0, 'g': 9.8}

    def teardown(self):

        try:
            shutil.rmtree(Scene.pydy_directory)
        except OSError:
            pass

        for json in glob.glob("*.json"):
            os.remove(json)



#!/usr/bin/env python

import os
import signal
import socket
import threading
import webbrowser

if os.name == 'nt':  # windows
    import win32api
    import win32con

from http.server import SimpleHTTPRequestHandler
from http.server import HTTPServer
raw_input = input

__all__ = ['Server']


class StoppableHTTPServer(HTTPServer):
    """
    Overrides BaseHTTPServer.HTTPServer to include a stop
    function.
    """

    def __init__(self, server_address, RequestHandlerClass,
                 bind_and_activate=True):
        self._event = threading.Event()
        HTTPServer.__init__(self, server_address,
                            RequestHandlerClass, bind_and_activate)

    def server_bind(self):
        HTTPServer.server_bind(self)
        self.socket.settimeout(1)
        self._event.set()

    def get_request(self):
        while self._event.is_set():
            try:
                sock, addr = self.socket.accept()
                sock.settimeout(None)
                return (sock, addr)
            except socket.timeout:
                pass

    def stop(self):
        self._event.clear()

    def serve(self):
        while self._event.is_set():
            try:
                self.handle_request()
            except TypeError:
                # When server is being closed, while loop can run once
                # after setting self._event = False depending on how it
                # is scheduled.
                pass

    @property
    def running(self):
        return self._event.is_set()


class Server(object):
    """
    Parameters
    ----------
    port : integer
        Defines the port on which the server will run. If this port is
        already bind, then it increment 1 until it finds a free port.
    scene_file : name of the scene_file generated for visualization
        A Valid PyDy generated scene file in 'directory' parameter.
    directory : absolute path of a directory
        Absolute path to the directory which contains scene_file with
        all other static files.

    Example
    -------
    >>> server = Server(scene_file=_scene_json_file)
    >>> server.run_server()

    """
    def __init__(self, scene_file, directory="static/", port=8000):
        self.scene_file = scene_file
        self.port = port
        self.directory = directory
        self.httpd = None
        self._thread = None
        self._sigint_handler = None

    def run_server(self, headless=False):
        # Change dir to static first.
        os.chdir(self.directory)
        print(os.getcwd())

        # Get a free port
        while self._check_port(self.port):
            self.port += 1

        handler_class = SimpleHTTPRequestHandler
        server_class = StoppableHTTPServer

        protocol = "HTTP/1.0"
        server_address = ('127.0.0.1', self.port)
        handler_class.protocol_version = protocol
        self.httpd = server_class(server_address, handler_class)
        sa = self.httpd.socket.getsockname()
        print("Serving HTTP on", sa[0], "port", sa[1], "...")
        print("To view visualization, open:\n")
        url = ("http://localhost:" + str(sa[1]) + "/index.html?load=" +
               self.scene_file)
        print(url)
        if not headless:
            webbrowser.open(url)

        print("Press Ctrl+C to stop server...")
        self._register_sigint_handler()
        self._thread = threading.Thread(target=self.httpd.serve)
        self._thread.start()

    def _windows_ctrl_handle(self, event):
        if event == win32con.CTRL_C_EVENT:
            self._stop_server(signal.SIGINT, None)
            return 1 # don't call other handlers
        return 0

    def _register_sigint_handler(self):
        if os.name == 'nt': # windows
            def handle(event):
                if event == win32con.CTRL_C_EVENT:
                    self._stop_server(signal.CTRL_C_EVENT, None)
                    return 1 # don't call other handlers
                return 0
            self._handle = handle
            win32api.SetConsoleCtrlHandler(self._handle, 1)
        else:
            self._sigint_handler = signal.getsignal(signal.SIGINT)
            signal.signal(signal.SIGINT, self._stop_server)

    def _check_port(self, port):
        soc = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        result = soc.connect_ex(('127.0.0.1', port))
        return result == 0

    def _stop_server(self, signalnum, frame):
        """
        Confirms and stops the visualization server
        signalnum:
            signal number
        frame:
            None or a frame object

        """
        # restore previous SIGINT handler
        if os.name == 'nt': # windows
            win32api.SetConsoleCtrlHandler(self._handle, 0)
        else:
            if self._sigint_handler is not None:
                signal.signal(signal.SIGINT, self._sigint_handler)
                self._sigint_handler = None

        res = raw_input("Shutdown this visualization server ([y]/n)? ")
        if not res or res[0].lower() == 'y':
            print("Shutdown confirmed")
            print("Shutting down server...")
            self.httpd.stop()
            self._thread.join()
        else:
            print("Resuming operations...")
            self._register_sigint_handler()



#!/usr/bin/env python

import os
import warnings
import tempfile
import shutil

import numpy as np
from numpy import testing
import sympy as sm
import sympy.physics.mechanics as me
from scipy.integrate import odeint
theano = sm.external.import_module('theano')
Cython = sm.external.import_module('Cython')

from ..system import System
from ..models import multi_mass_spring_damper, n_link_pendulum_on_cart
from ..utils import PyDyImportWarning, sympy_newer_than

SYMPY_VERSION = sm.__version__

warnings.simplefilter('once', PyDyImportWarning)


class TestSystem():

    def setup(self):

        # Create a simple system with one specified quantity.
        self.sys = multi_mass_spring_damper(1, apply_gravity=True,
                                            apply_external_forces=True)
        self.specified_symbol = next(iter(self.sys.specifieds_symbols))
        self.constant_map = dict(zip(sm.symbols('m0, k0, c0, g'),
                                     [2.0, 1.5, 0.5, 9.8]))
        self.sys.specifieds = {self.specified_symbol: np.ones(1)}
        self.sys.constants = self.constant_map
        self.kane = self.sys.eom_method

        # Create a system with multiple specified quantities.
        self.kane_nlink = n_link_pendulum_on_cart(3, cart_force=True,
                                                  joint_torques=True).eom_method

    def test_init(self):

        # Check defaults for most attributes.
        # -----------------------------------
        sys = System(self.kane)

        assert (sys.constants_symbols ==
                set(sm.symbols('k0, m0, g, c0')))
        assert sys.specifieds_symbols == {self.specified_symbol}
        assert sys.states == me.dynamicsymbols('x0, v0')
        assert sys.evaluate_ode_function is None
        assert sys.eom_method is self.kane
        assert sys.ode_solver is odeint
        assert sys.specifieds == dict()
        assert sys.initial_conditions == dict()
        assert sys.constants == dict()
        assert sys.times == list()

        # Specify a bunch of attributes during construction.
        # --------------------------------------------------
        ic = {me.dynamicsymbols('x0'): 3.6, me.dynamicsymbols('v0'): 4.3}
        sys = System(self.kane,
                     ode_solver=odeint,
                     specifieds={self.specified_symbol: np.ones(1)},
                     initial_conditions=ic,
                     constants=self.constant_map)

        assert sys.eom_method is self.kane
        assert list(sys.specifieds.keys()) == [me.dynamicsymbols('f0')]
        testing.assert_allclose(list(sys.specifieds.values()),
                                [np.ones(1)])
        assert sys.initial_conditions.keys() == ic.keys()
        testing.assert_allclose(list(sys.initial_conditions.values()),
                                list(ic.values()))
        assert sys.constants.keys() == self.constant_map.keys()
        testing.assert_allclose(list(sys.constants.values()),
                                list(self.constant_map.values()))

        # Use old specifieds.
        # -------------------
        sys = System(self.kane,
                     ode_solver=odeint,
                     specifieds={'symbols': [self.specified_symbol],
                                 'values': np.ones(1)},
                     initial_conditions=ic,
                     constants=self.constant_map)

    def test_coordinates(self):
        assert self.sys.coordinates == self.kane.q[:]

    def test_speeds(self):
        assert self.sys.speeds == self.kane.u[:]

    def test_states(self):
        assert self.sys.states == self.kane.q[:] + self.kane.u[:]

    def test_constants(self):

        # User-specified numerical values for constants.
        constants = {sm.symbols('m0'): 3.0, sm.symbols('c0'): 4.5}

        # The System should fill in the remaining constants with the defaults.

        # Construct a new system with our constants at construction time.
        # ---------------------------------------------------------------
        sys = System(self.kane, constants=constants)

        assert sys.constants.keys() == constants.keys()
        testing.assert_allclose(list(sys.constants.values()),
                                list(constants.values()))

        # Set constants after construction.
        # ---------------------------------
        sys = System(self.kane)

        # All constants have the default value.
        assert sys.constants == dict()
        sys.constants = constants

        assert sys.constants.keys() == constants.keys()
        testing.assert_allclose(list(sys.constants.values()),
                                list(constants.values()))

        # Using the property as a dict.
        # -----------------------------
        sys = System(self.kane)
        # Modifying the dict directly does change the dict.
        sys.constants[sm.symbols('m0')] = 9.3
        assert list(sys.constants.keys()) == [sm.symbols('m0')]
        testing.assert_allclose(list(sys.constants.values()), [9.3])

        # Putting in a non-constant key does not raise exception.
        sys.constants[me.dynamicsymbols('v0')] = 9.8
        # Then, if we integrate, we do error-checking and we get an exception.
        sys.times = [0.0, 1.0]
        with testing.assert_raises(ValueError):
            sys.integrate()

        # Provide a constant that isn't actually a constant.
        # --------------------------------------------------
        with testing.assert_raises(ValueError):
            sys.constants = {me.dynamicsymbols('x0'): 1.3}
        with testing.assert_raises(ValueError):
            sys.constants = {me.dynamicsymbols('f0'): 1.8}

    def test_specifieds(self):

        sys = System(self.kane)
        assert sys.specifieds == dict()
        sys.specifieds = {me.dynamicsymbols('f0'): 5.9}
        assert list(sys.specifieds.keys()) == [me.dynamicsymbols('f0')]
        testing.assert_allclose(list(sys.specifieds.values()), [5.9])

        # Using the property as a dict.
        # -----------------------------
        # Modifying the dict directly does change the dict.
        sys.specifieds[me.dynamicsymbols('f0')] = 5.1
        assert list(sys.specifieds.keys()) == [me.dynamicsymbols('f0')]
        testing.assert_allclose(list(sys.specifieds.values()), [5.1])
        # Putting in a non-specified key does not raise exception.
        sys.specifieds[me.dynamicsymbols('v0')] = 3.5
        # Then, if we integrate, we do error-checking and we get an exception.
        sys.times = [0.0, 1.0]
        with testing.assert_raises(ValueError):
            sys.integrate()

        sys = System(self.kane)
        # Putting in a value of the wrong length does not raise exception.
        sys.specifieds[me.dynamicsymbols('f0')] = 3.1 * np.ones(2)
        # Then, if we integrate, we do error-checking and we get an exception.
        # TODO actually, this does not seem to throw an exception.
        # TODO with testing.assert_raises(ValueError):
        # TODO     sys.integrate([0.0, 1.0])

        # The specified symbol must exist in the equations of motion and not
        # be a state.
        # ------------------------------------------------------------------
        sys = System(self.kane)
        with testing.assert_raises(ValueError):
            sys.specifieds = {sm.symbols('m0'): 5.4}
        with testing.assert_raises(ValueError):
            sys.specifieds = {me.dynamicsymbols('x0'): 5.1}

        # Complex error-checking when using property as a dict.
        # -----------------------------------------------------
        sys = System(self.kane_nlink)
        spec_syms = list(sys.specifieds_symbols)
        times = np.linspace(0, 0.5, 10)
        sys.specifieds = {
            spec_syms[0]: lambda x, t: np.ones(t),
            (spec_syms[3], spec_syms[1]): lambda x, t: np.array([4, 2]),
            spec_syms[2]: 3.0 * np.ones(1)}
        # These won't throw an exception b/c we're modifying the dict directly.
        sys.specifieds[spec_syms[1]] = 7.1
        sys.times = times
        # This does.
        with testing.assert_raises(ValueError):
            sys.integrate()

        sys = System(self.kane_nlink)
        # This puts too many entries in the dict.
        sys.specifieds[spec_syms[0]] = 3.7
        sys.specifieds[(spec_syms[0], spec_syms[3])] = 5.8
        sys.times = times
        with testing.assert_raises(ValueError):
            sys.integrate()

        # This gets rid of the previous default entries, and should work
        # properly.
        # Also check here that optional arguments can be passed to the solver.
        sys.specifieds.pop(spec_syms[0])
        state_traj, infodict = sys.integrate(full_output=True)

        # Test old way of providing specifieds.
        # -------------------------------------
        sys = System(self.kane_nlink)
        spec_syms = list(sys.specifieds_symbols)
        # Get numbers using the new way.
        sys.specifieds = dict(zip(spec_syms, [1.0, 2.0, 3.0, 4.0]))
        sys.times = times
        x_01 = sys.integrate()

        # Now use the old way.
        sys.specifieds = {'symbols': spec_syms,
                          'values': [1.0, 2.0, 3.0, 4.0]}
        x_02 = sys.integrate()
        testing.assert_allclose(x_01, x_02)

        # Error checks for the new way.
        # -----------------------------
        with testing.assert_raises(ValueError):
            sys.specifieds = {'symbols': [sm.symbols('m1')], 'values': [1.0]}
        with testing.assert_raises(ValueError):
            sys.specifieds = {'symbols': [sm.symbols('T2, T2')],
                              'values': [1, 2]}
        with testing.assert_raises(ValueError):
            sys.specifieds = {'symbols': [me.dynamicsymbols('T2')],
                              'values': [1.0]}

        # Reordering causes issues!
        # -------------------------
        sys.specifieds = {
            'symbols': [spec_syms[1], spec_syms[0], spec_syms[2],
                        spec_syms[3]],
            'values': [2.0, 1.0, 3.0, 4.0]}
        # I tested: x_01 is not allclose to x_03.

        sys.generate_ode_function()
        x_04 = sys.integrate()
        testing.assert_allclose(x_01, x_04)

        # Test with no specifieds.
        sys = multi_mass_spring_damper(1, apply_gravity=True)
        sys.initial_conditions = {me.dynamicsymbols('x0'): 0.1,
                                  me.dynamicsymbols('v0'): -1.0}
        sys.times = times
        sys.integrate()

    def test_times(self):
        times1 = [0, 1, 2, 3, 4, 5, 6, 7, 8, 9]
        times2 = [0, -2, 7, 3, -5]
        times3 = [1, 2, 7, 4, 5]
        times4 = 4

        sys = System(self.kane, times=times1)
        testing.assert_allclose(sys.times, times1)

        with testing.assert_raises(ValueError):
            sys.times = times2

        with testing.assert_raises(ValueError):
            sys.times = times3

        with testing.assert_raises(TypeError):
            sys.times = times4

    def test_ode_solver(self):

        assert self.sys.ode_solver == odeint
        self.sys.ode_solver = max
        assert self.sys.ode_solver is max

        # ode_solver must be a function
        # -----------------------------
        with testing.assert_raises(ValueError):
            self.sys.ode_solver = 5

    def test_initial_conditions(self):

        # Partially provided ic's.
        ic = {me.dynamicsymbols('v0'): 6.1}

        # Using the constructor.
        # ----------------------
        sys = System(self.kane, initial_conditions=ic)
        assert sys.initial_conditions.keys() == ic.keys()
        testing.assert_allclose(list(sys.initial_conditions.values()),
                                list(ic.values()))

        # Set the attribute.
        # ------------------
        sys = System(self.kane)
        sys.initial_conditions = ic
        assert sys.initial_conditions.keys() == ic.keys()
        testing.assert_allclose(list(sys.initial_conditions.values()),
                                list(ic.values()))

        # Using the property as a dict.
        # -----------------------------
        # Modifying hte dict directly does change the dict.
        sys = System(self.kane, times=[0.0, 1.0])
        sys.initial_conditions[me.dynamicsymbols('x0')] = 5.8
        assert list(sys.initial_conditions.keys()) == [me.dynamicsymbols('x0')]
        testing.assert_allclose(list(sys.initial_conditions.values()), [5.8])
        # Putting in a non-state key does not raise exception.
        sys.initial_conditions[sm.symbols('m0')] = 7.9
        # Then, if we integrate, we do error-checking and we get an exception.

        with testing.assert_raises(ValueError):
            sys.integrate()

        # Keys must be coords or speeds.
        # ------------------------------
        with testing.assert_raises(ValueError):
                self.sys.initial_conditions = {sm.symbols('k0'): 0.4}
        with testing.assert_raises(ValueError):
                self.sys.initial_conditions = {sm.symbols('f0'): 7.3}

    def test_generate_ode_function(self):

        rhs = self.sys.generate_ode_function()

        assert rhs is self.sys.evaluate_ode_function

        args = (self.sys.specifieds, self.sys.constants)

        actual = rhs(np.ones(2), 0.0, *args)

        # Regression.
        testing.assert_allclose(actual, np.array([1, 9.3]))

        # n-link cart: play with specifieds.
        # ----------------------------------
        sys = System(self.kane_nlink)
        spec_syms = list(sys.specifieds_symbols)
        rhs = sys.generate_ode_function()
        x = np.array(np.random.random(len(sys.states)))
        args = (self.sys.specifieds,
                {k: 1.0 for k in sys.constants_symbols})

        # Specify constants in two different ways and ensure we get the
        # same results. This is like Jason's test in codegen.
        args = (dict(zip(spec_syms, [1.0, 2.0, 3.0, 4.0])),
                {k: 1.0 for k in sys.constants_symbols})
        xd_01 = rhs(x, 0.0, *args)

        args = ({spec_syms[0]: lambda x, t: np.ones(1),
                 (spec_syms[3], spec_syms[1]): lambda x, t: np.array([4, 2]),
                 spec_syms[2]: 3.0 * np.ones(1)},
                {k: 1.0 for k in sys.constants_symbols})
        xd_02 = rhs(x, 0.0, *args)

        testing.assert_allclose(xd_01, xd_02)

    def test_integrate(self):

        times = np.linspace(0, 1, 100)

        # Try without calling generate_ode_function.
        # ------------------------------------------
        sys = System(self.kane, times=times)
        x_01 = sys.integrate()

        sys = System(self.kane, times=times)
        sys.generate_ode_function(generator='lambdify')
        x_02 = sys.integrate()

        testing.assert_allclose(x_01, x_02)

        # Ensure that the defaults are as expected.
        # -----------------------------------------
        constants_dict = dict(zip(sm.symbols('m0, k0, c0, g'),
                                  [1.0, 1.0, 1.0, 1.0]))
        specified_dict = {me.dynamicsymbols('f0'): 0.0}
        x_03 = sys.ode_solver(sys.evaluate_ode_function, [0, 0], sys.times,
                              args=(specified_dict, constants_dict))
        testing.assert_allclose(x_02, x_03)

        # Ensure that initial conditions are reordered properly.
        # ------------------------------------------------------
        sys = System(self.kane, times=times)
        # I know that this is the order of the states.
        x0 = [5.1, 3.7]
        ic = {me.dynamicsymbols('x0'): x0[0], me.dynamicsymbols('v0'): x0[1]}
        sys.initial_conditions = ic
        x_04 = sys.integrate()
        x_05 = sys.ode_solver(
            sys.evaluate_ode_function, x0, sys.times,
            args=(sys._specifieds_padded_with_defaults(),
                  sys._constants_padded_with_defaults()))

        testing.assert_allclose(x_04, x_05)

        # Test a generator other than lambdify.
        # -------------------------------------
        if theano:
            sys.generate_ode_function(generator='theano')
            sys.times = times
            x_06 = sys.integrate()
            testing.assert_allclose(x_04, x_06)
        else:
            warnings.warn("Theano was not found so the related tests are being"
                          " skipped.", PyDyImportWarning)

        # Unrecognized generator.
        # -----------------------
        sys = System(self.kane, times=times)
        with testing.assert_raises(NotImplementedError):
            sys.generate_ode_function(generator='made-up')

        # Test pass kwargs to the generators.
        if Cython:
            self.tempdirpath = tempfile.mkdtemp()
            prefix = 'my_test_file'
            self.sys.generate_ode_function(generator='cython',
                                           prefix=prefix,
                                           tmp_dir=self.tempdirpath)
            assert [True for f in os.listdir(self.tempdirpath)
                    if f.startswith(prefix)]
        else:
            warnings.warn("Cython was not found so the related tests are being"
                          " skipped.", PyDyImportWarning)
    def cleanup(self):
        shutil.rmtree(self.tempdirpath)


def test_specifying_coordinate_issue_339():
    """This test ensures that you can use derivatives as specified values."""

    # beta will be a specified angle
    beta = me.dynamicsymbols('beta')
    q1, q2, q3, q4 = me.dynamicsymbols('q1, q2, q3, q4')
    u1, u2, u3, u4 = me.dynamicsymbols('u1, u2, u3, u4')

    N = me.ReferenceFrame('N')
    A = N.orientnew('A', 'Axis', (q1, N.x))
    B = A.orientnew('B', 'Axis', (beta, A.y))

    No = me.Point('No')
    Ao = No.locatenew('Ao', q2 * N.x + q3 * N.y + q4 * N.z)
    Bo = Ao.locatenew('Bo', 10 * A.x + 10 * A.y + 10 * A.z)

    A.set_ang_vel(N, u1 * N.x)
    B.ang_vel_in(N)  # compute it automatically

    No.set_vel(N, 0)
    Ao.set_vel(N, u2 * N.x + u3 * N.y + u4 * N.z)
    Bo.v2pt_theory(Ao, N, B)

    body_A = me.RigidBody('A', Ao, A, 1.0, (me.inertia(A, 1, 2, 3), Ao))
    body_B = me.RigidBody('B', Bo, B, 1.0, (me.inertia(A, 3, 2, 1), Bo))

    bodies = [body_A, body_B]
    # TODO : This should be able to be simple an empty iterable.
    loads = [(No, 0 * N.x)]

    kdes = [u1 - q1.diff(),
            u2 - q2.diff(),
            u3 - q3.diff(),
            u4 - q4.diff()]

    kane = me.KanesMethod(N, q_ind=[q1, q2, q3, q4],
                          u_ind=[u1, u2, u3, u4], kd_eqs=kdes)

    if sympy_newer_than('1.0'):
        fr, frstar = kane.kanes_equations(bodies, loads)
    else:
        fr, frstar = kane.kanes_equations(loads, bodies)

    sys = System(kane)

    sys.specifieds = {(beta, beta.diff(), beta.diff().diff()):
                      lambda x, t: np.array([1.0, 1.0, 1.0])}

    sys.times = np.linspace(0, 10, 20)

    sys.integrate()



from sympy.matrices.expressions import Identity

from .visualization_frame import VisualizationFrame
from ..utils import sympy_equal_to_or_newer_than

__all__ = ['PointLight']

class PointLight(VisualizationFrame):
    """
    Creates a PointLight for the visualization
    The PointLight is inherited from VisualizationFrame,

    It can also be attached to dynamics objects, hence we can
    get a moving Light. All the transformation matrix generation
    methods are applicable to a PointLight.
    Like VisualizationFrame,
    It can also be initialized using:
    1)Rigidbody
    2)ReferenceFrame, Point
    3)ReferenceFrame, Particle
    Either one of these must be supplied during initialization

    Unlike VisualizationFrame, It doesnt require a Shape argument.
    """

    def __init__(self, *args, **kwargs):
        """
        Initialises a PointLight object.
        To initialize a point light, we need to supply
        a name(optional), a reference frame, and a point.


        Parameters
        ==========
        name : str, optional
            Name assigned to VisualizationFrame, default is unnamed
        reference_frame : ReferenceFrame
            A reference_frame with respect to which all orientations of the
            shape takes place, during visualizations/animations.
        origin : Point
            A point with respect to which all the translations of the shape
            takes place, during visualizations/animations.
        rigidbody : RigidBody
            A rigidbody whose reference frame and mass center are to be
            assigned as reference_frame and origin of the
            VisualizationFrame.
        particle : Particle
            A particle whose point is assigned as origin of the
            VisualizationFrame.

        Examples
        ========
        >>> from pydy.viz import PointLight
        >>> from sympy.physics.mechanics import \
                               ReferenceFrame, Point, RigidBody, \
                                Particle, inertia
        >>> from sympy import symbols
        >>> I = ReferenceFrame('I')
        >>> O = Point('O')
        >>> #initializing with reference frame, point
        >>> light = PointLight('light', I, O)
        >>> Ixx, Iyy, Izz, mass = symbols('Ixx Iyy Izz mass')
        >>> i = inertia(I, Ixx, Iyy, Izz)
        >>> rbody = RigidBody('rbody', O, I, mass, (inertia, O))
        >>> # Initializing with a rigidbody ..
        >>> light = PointLight('frame2', rbody)
        >>> Pa = Particle('Pa', O, mass)
        >>> #initializing with Particle, reference_frame ...
        >>> light = PointLight('frame3', I, Pa)
        """

        try:
            self._color = kwargs['color']
        except KeyError:
            self._color = 'white'

        #Now we use same approach as in VisualizationFrame
        #for setting reference_frame and origin
        i = 0
        #If first arg is not str, name the visualization frame 'unnamed'
        if isinstance(args[i], str):
            self._name = args[i]
            i += 1
        else:
            self._name = 'unnamed'

        try:
            if sympy_equal_to_or_newer_than('1.0'):
                self._reference_frame = args[i].frame
            else:
                self._reference_frame = args[i].get_frame()

            self._origin = args[i].masscenter

        except AttributeError:
            #It is not a rigidbody, hence this arg should be a
            #reference frame
            try:
                dcm = args[i]._dcm_dict
                self._reference_frame = args[i]
                i += 1
            except AttributeError:
                raise TypeError(''' A ReferenceFrame is to be supplied
                                   before a Particle/Point. ''')

            #Now next arg can either be a Particle or point
            try:
                self._origin = args[i].point
            except AttributeError:
                self._origin = args[i]

        #basic thing required, transform matrix
        self._transform = Identity(4).as_mutable()

    def __str__(self):
        return 'PointLight: ' + self._name

    def __repr__(self):
        return 'PointLight'

    @property
    def color(self):
        """
        Color of Light.
        """
        return self._color

    @color.setter
    def color(self, new_color):
        """
        Sets the color of Light.

        """
        if not isinstance(new_color, str):
            raise TypeError('''Color should be a valid str object''')
        else:
            self._color = new_color

    def color_in_rgb(self):
        """
        Returns the rgb value of the
        defined light color.
        """
        return self._color_rgb

    def generate_scene_dict(self):
        """
        This method generates information for a static
        visualization in the initial conditions, in the form
        of dictionary. This contains light parameters followed
        by an init_orientation Key.

        Before calling this method, all the transformation matrix
        generation methods should be called, or it will give an error.
        Returns
        =======
        A dict with following Keys:

        1. name: name for the camera
        2. color: Color of the light
        3. init_orientation: Initial orientation
           of the light object

        """
        scene_dict = { id(self): {} }
        scene_dict[id(self)]['name'] = self.name
        scene_dict[id(self)]['type'] = self.__repr__()
        scene_dict[id(self)]['color'] = self.color
        scene_dict[id(self)]["simulation_id"] = id(self)
        scene_dict[id(self)]["init_orientation"] = self._visualization_matrix[0]

        return scene_dict

    def generate_simulation_dict(self):
        """
        Generates the simulation information for this Light object.
        It maps the simulation data information to the
        scene information via a unique id.

        Before calling this method, all the transformation matrix
        generation methods should be called, or it will give an error.

        Returns
        =======

        A dictionary containing list of 4x4 matrices mapped to
        the unique id as the key.

        """
        simulation_dict = {}
        try:
            simulation_dict[id(self)] = self._visualization_matrix

        except:
            raise RuntimeError("Please call the numerical ",
                               "transformation methods, ",
                               "before generating visualization dict")


        return simulation_dict



print("import: 'pydy'")
import pydy

print("import: 'pydy.codegen'")
import pydy.codegen

print("import: 'pydy.codegen.tests'")
import pydy.codegen.tests

print("import: 'pydy.models'")
import pydy.models

print("import: 'pydy.system'")
import pydy.system

print("import: 'pydy.utils'")
import pydy.utils

print("import: 'pydy.viz'")
import pydy.viz

print("import: 'pydy.viz.tests'")
import pydy.viz.tests




#!/usr/bin/env python

from pkg_resources import parse_version
from setuptools import __version__ as SETUPTOOLS_VERSION
from nose.tools import assert_raises

from sympy import cos, sin, tan, sqrt, Matrix
from sympy.physics.mechanics import dynamicsymbols

from ..utils import sympy_equal_to_or_newer_than, wrap_and_indent
from ..codegen.cython_code import CythonMatrixGenerator


def test_sympy_equal_to_or_newer_than():

    # sympy_equal_to_or_newer_than(version, installed_version)
    assert sympy_equal_to_or_newer_than('0.7.6.dev', '0.7.6.dev')
    assert not sympy_equal_to_or_newer_than('0.7.6', '0.7.6.dev')
    assert sympy_equal_to_or_newer_than('0.7.5', '0.7.6.dev')
    assert sympy_equal_to_or_newer_than('0.6.5', '0.7.6.dev')
    assert not sympy_equal_to_or_newer_than('0.7.7', '0.7.6.dev')
    if parse_version(SETUPTOOLS_VERSION) >= parse_version('8.0'):
        with assert_raises(ValueError):
            sympy_equal_to_or_newer_than('0.7.7', '0.7.6-git')


def test_codegen_linewrap():

    # Generated can result in long expressions with no obvious place to insert a
    # newline. Refer to https://github.com/pydy/pydy/issues/263.
    x, y, z = dynamicsymbols('x y z')
    expr = (x*y*z*cos(x)*sin(x)*tan(x)*sqrt(x)*cos(y)*sin(y)*tan(y)*
            sqrt(y)*cos(z)*sin(z)*tan(z)*sqrt(z)*cos(x*y)*sin(x*y)*tan(x*y)*
            sqrt(x*y)* cos(y*z)*sin(y*z)*tan(y*z)*sqrt(y*z)*cos(z*x)*
            sin(z*x)*tan(z*x)*sqrt(z*x)*3)/8
    mat_expr = Matrix([expr])

    q = [x, y, z]
    # Don't raise an error with this line.
    gen = CythonMatrixGenerator([q], [mat_expr])


def test_wrap_and_indent():

    lines = ["a + b + c + d + e", "a + b + c + d + e"]
    wrapped = wrap_and_indent(lines, width=10)
    expected = """\
    a + b
    + c +
    d + e
    a + b
    + c +
    d + e"""
    assert wrapped == expected

    wrapped = wrap_and_indent(lines, width=14, continuation=' ...')
    expected = """\
    a + b ...
    + c + ...
    d + e
    a + b ...
    + c + ...
    d + e"""
    assert wrapped == expected

    lines = ["% a + b + c + d + e", "% a + b + c + d + e"]
    wrapped = wrap_and_indent(lines, width=12, comment='%')
    expected = """\
    % a + b
    % + c + d
    % + e
    % a + b
    % + c + d
    % + e"""
    assert wrapped == expected



__all__ = ['VisualizationFrame']

import sys
from collections.abc import Iterator
import numpy as np
from sympy import Dummy, lambdify
from sympy.matrices.expressions import Identity
from sympy.physics.mechanics import Point, ReferenceFrame
try:
    import pythreejs as p3js
except ImportError:
    p3js = None

from .shapes import Shape
from ..utils import sympy_equal_to_or_newer_than


class VisualizationFrame(object):
    """
    A VisualizationFrame represents an object that you want to visualize.
    It allows you to easily associate a reference frame and a point
    with a shape.

    A VisualizationFrame can be attached to only one Shape Object.
    It can be nested, i.e we can add/remove multiple visualization frames to
    one visualization frame. On adding the parent frame to the
    Scene object, all the children of the parent visualization frame
    are also added, and hence can be visualized and animated.

    A VisualizationFrame needs to have a ReferenceFrame, and a Point
    for it to form transformation matrices for visualization and
    animations.

    The ReferenceFrame and Point are required to be provided during
    initialization. They can be supplied in the form of any one of these:

    1)reference_frame, point argument.
    2)a RigidBody argument
    3)reference_frame, particle argument.

    In addition to these arguments, A shape argument is also required.
    """
    def __init__(self, *args):
        """
        To initialize a visualization frame a ReferenceFrame,
        Point, and Shape are required. These ReferenceFrame
        and Point can be passed provided in three ways:

        1) RigidBody: the RigidBody's frame and mass center are used.
        2) ReferenceFrame and a Particle: The Particle's Point is used.
        3) ReferenceFrame and a Point

        Parameters
        ==========
        name : str, optional
            Name assigned to VisualizationFrame, default is unnamed
        reference_frame : ReferenceFrame
            A reference_frame with respect to which all orientations of the
            shape takes place, during visualizations/animations.
        origin : Point
            A point with respect to which all the translations of the shape
            takes place, during visualizations/animations.
        rigidbody : RigidBody
            A rigidbody whose reference frame and mass center are to be
            assigned as reference_frame and origin of the
            VisualizationFrame.
        particle : Particle
            A particle whose point is assigned as origin of the
            VisualizationFrame.
        shape : Shape
            A shape to be attached to the VisualizationFrame

        Examples
        ========
        >>> from pydy.viz import VisualizationFrame, Sphere
        >>> from sympy.physics.mechanics import \
                               ReferenceFrame, Point, RigidBody, \
                                Particle, inertia
        >>> from sympy import symbols
        >>> I = ReferenceFrame('I')
        >>> O = Point('O')
        >>> shape = Sphere(5)
        >>> #initializing with reference frame, point
        >>> frame1 = VisualizationFrame('frame1', I, O, shape)
        >>> Ixx, Iyy, Izz, mass = symbols('Ixx Iyy Izz mass')
        >>> i = inertia(I, Ixx, Iyy, Izz)
        >>> rbody = RigidBody('rbody', O, I, mass, (inertia, O))
        >>> # Initializing with a rigidbody ..
        >>> frame2 = VisualizationFrame('frame2', rbody, shape)
        >>> Pa = Particle('Pa', O, mass)
        >>> #initializing with Particle, reference_frame ...
        >>> frame3 = VisualizationFrame('frame3', I, Pa, shape)
        """
        #Last arg should be a Shape ..
        if isinstance(args[-1], Shape):
            self._shape = args[-1]
        else:
            raise TypeError("Please provide a valid shape object as the last "
                            " positional argument.")
        i = 0
        # If first arg is not str, name the visualization frame 'unnamed'
        if isinstance(args[i], str):
            self.name = args[i]
            i += 1
        else:
            self.name = 'unnamed'

        try:
            if sympy_equal_to_or_newer_than('1.0'):
                self.reference_frame = args[i].frame
            else:
                self.reference_frame = args[i].get_frame()
            self.origin = args[i].masscenter

        except AttributeError:
            #It is not a rigidbody, hence this arg should be a
            #reference frame
            try:
                dcm = args[i]._dcm_dict
                self.reference_frame = args[i]
                i += 1
            except AttributeError:
                raise TypeError(''' A ReferenceFrame is to be supplied
                                   before a Particle/Point. ''')

            #Now next arg can either be a Particle or point
            try:
                self.origin = args[i].point

            except AttributeError:
                self.origin = args[i]

    #setting attributes ..
    def __str__(self):
        return 'VisualizationFrame ' + self.name

    def __repr__(self):
        return 'VisualizationFrame'

    @property
    def name(self):
        """
        Name of the VisualizationFrame.
        """
        return self._name

    @name.setter
    def name(self, new_name):
        """
        Sets the name of the VisualizationFrame.

        """
        if not isinstance(new_name, str):
            raise TypeError('''Name should be a str object''')
        else:
            self._name = new_name

    @property
    def origin(self):
        """
        Origin of the VisualizationFrame,
        with respect to which all translational transformations
        take place.
        """
        return self._origin

    @origin.setter
    def origin(self, new_origin):
        """
        Sets the origin of the VisualizationFrame.
        """
        if not isinstance(new_origin, Point):
            raise TypeError('''origin should be a valid Point Object''')
        else:
            self._origin = new_origin

    @property
    def reference_frame(self):
        """
        reference_frame of the VisualizationFrame,
        with respect to which all rotational/orientational
        transformations take place.
        """
        return self._reference_frame

    @reference_frame.setter
    def reference_frame(self, new_reference_frame):
        if not isinstance(new_reference_frame, ReferenceFrame):
            raise TypeError('''reference_frame should be a valid
                                ReferenceFrame object.''')
        else:
            self._reference_frame = new_reference_frame

    @property
    def shape(self):
        """
        shape in the VisualizationFrame.
        A shape attached to the visualization frame.
        NOTE: Only one shape can be attached to a visualization frame.
        """
        return self._shape

    @shape.setter
    def shape(self, new_shape):
        """
        Sets the shape for VisualizationFrame.
        """
        if not isinstance(new_shape, Shape):
            raise TypeError('''shape should be a valid Shape object.''')
        else:
            self._shape = new_shape

    def generate_transformation_matrix(self, reference_frame, point):
        """Generates a symbolic transformation matrix, with respect to the
        provided reference frame and point.

        Parameters
        ==========
        reference_frame : ReferenceFrame
            A reference_frame with respect to which transformation matrix is
            generated.
        point : Point
            A point with respect to which transformation matrix is
            generated.

        Returns
        =======
        A 4 x 4 SymPy matrix, containing symbolic expressions describing the
        transformation as a function of time.

        """
        rotation_matrix = self.reference_frame.dcm(reference_frame)
        self._transform = Identity(4).as_mutable()
        self._transform[:3, :3] = rotation_matrix

        point_vector = self.origin.pos_from(point)
        try:
            self._transform[3, :3] = point_vector.to_matrix(reference_frame).T
        except AttributeError:
            # In earlier versions of sympy, 'Vector' object has no attribute
            # 'to_matrix'.
            self._transform[3, 0] = point_vector.dot(reference_frame.x)
            self._transform[3, 1] = point_vector.dot(reference_frame.y)
            self._transform[3, 2] = point_vector.dot(reference_frame.z)
        return self._transform

    def generate_numeric_transform_function(self, dynamic_variables,
                                            constant_variables):
        """Returns a function which can compute the numerical values of the
        transformation matrix given the numerical dynamic variables (i.e.
        functions of time or states) and the numerical system constants.


        Parameters
        ==========
        dynamic_variables : list of sympy.Functions(time)
            All of the dynamic symbols used in defining the orientation and
            position of this visualization frame.
        constant_variables : list of sympy.Symbols
            All of the constants used in defining the orientation and
            position of this visualization frame.

        Returns
        =======
        numeric_transform : list of functions
            A list of functions which return the numerical transformation
            for each element in the transformation matrix.

        """

        dummy_symbols = [Dummy() for i in dynamic_variables]
        dummy_dict = dict(zip(dynamic_variables, dummy_symbols))
        transform = self._transform.subs(dummy_dict).reshape(16, 1)
        dummy_symbols.extend(constant_variables)

        # Create a numeric transformation for each element in the transformation
        # matrix. We cannot lambdify the transformation matrix as calling
        # lambdify of a constant expression returns a scalar, even if the
        # lambdify function arguments are sequences:
        # https://github.com/sympy/sympy/issues/5642
        self._numeric_transform = []
        for i in range(16):
            t = transform[i]
            if t.has(Dummy):
                f = lambdify(dummy_symbols, t, modules='numpy')
            else:
                f = lambdify(constant_variables, t, modules='numpy')
            self._numeric_transform.append(f)
        return self._numeric_transform

    def evaluate_transformation_matrix(self, dynamic_values, constant_values):
        """Returns the numerical transformation matrices for each time step.

        Parameters
        ----------
        dynamic_values : array_like, shape(m,) or shape(n, m)
            The m state values for each n time step.
        constant_values : array_like, shape(p,)
            The p constant parameter values of the system.

        Returns
        -------
        transform_matrix : numpy.array, shape(n, 16)
            A 4 x 4 transformation matrix for each time step.

        """
        #If states is instance of numpy array, well and good.
        #else convert it to one:

        states = np.squeeze(np.array(dynamic_values))
        if not isinstance(constant_values, Iterator):
            constant_values = list(constant_values)
        if len(states.shape) > 1:
            n = states.shape[0]
            args = []
            for a in np.split(states, states.shape[1], 1):
                args.append(np.squeeze(a))
            for a in constant_values:
                args.append(np.repeat(a, n))
        else:
            n = 1
            args = np.hstack((states, constant_values))

        new = np.zeros((n, 16))
        for i, t in enumerate(self._numeric_transform):
            if callable(t):
                try:
                    new[:, i] = t(*args)
                except TypeError:
                    # dynamic values are not necessary so pass only constant
                    # values into transform function
                    new[:, i] = np.repeat(t(*constant_values), n)
            else:
                new[:, i] = np.repeat(t, n)
        self._visualization_matrix = new.tolist()
        return self._visualization_matrix

    def generate_scene_dict(self, constant_map={}):
        """
        This method generates information for a static
        visualization in the initial conditions, in the form
        of dictionary. This contains shape information
        from `Shape.generate_dict()` followed by an
        init_orientation Key.

        Before calling this method, all the transformation matrix
        generation methods should be called, or it will give an error.

        Parameters
        ==========
        constant_map : dictionary
            Constant map is required when Shape contains sympy expressions.This
            dictionary maps sympy expressions/symbols to numerical values(floats)

        Returns
        =======
        A dictionary built with a call to `Shape.generate_dict`.
        Additional keys included in the dict are following:

        1. init_orientation: Specifies the initial orientation
           of the `VisualizationFrame`.
        2. reference_frame_name: Name(str) of the reference_frame
           attached to this VisualizationFrame.
        3. simulation_id: an arbitrary integer to map scene description
           with the simulation data.


        """
        scene_dict = { id(self): {} }
        scene_dict[id(self)] = self.shape.generate_dict(constant_map=constant_map)
        scene_dict[id(self)]['name'] = self.name
        scene_dict[id(self)]["reference_frame_name"] = str(self.reference_frame)
        scene_dict[id(self)]["simulation_id"] = id(self)

        try:
            scene_dict[id(self)]["init_orientation"] = self._visualization_matrix[0]
        except:
            raise RuntimeError("Cannot generate visualization data " + \
                                "because numerical transformation " + \
                               "has not been performed, " + \
                                "Please call the numerical " + \
                               "transformation methods, " + \
                               "before generating visualization dict")

        return scene_dict

    def generate_simulation_dict(self):
        """
        Generates the simulation information for this visualization
        frame. It maps the simulation data information to the
        scene information via a unique id.

        Before calling this method, all the transformation matrix
        generation methods should be called, or it will give an error.

        Returns
        =======

        A dictionary containing list of 4x4 matrices mapped to
        the unique id as the key.

        """
        simulation_dict = {}
        try:
            simulation_dict[id(self)] = self._visualization_matrix
        except:
            raise RuntimeError("Cannot generate visualization data "
                               "because numerical transformation "
                               "has not been performed, "
                               "Please call the numerical "
                               "transformation methods, "
                               "before generating visualization dict")

        return simulation_dict

    def _create_keyframetrack(self, times, dynamic_values, constant_values,
                              constant_map=None):
        """Sets attributes with a Mesh and KeyframeTrack for animating this
        visualization frame.

        Parameters
        ==========
        times : ndarray, shape(n,)
            Array of monotonically increasing or decreasing values of time.
        dynamics_values : ndarray, shape(n, m)
            Array of state values for each time.
        constant_values : array_like, shape(p,)
            Array of values for the constants.
        constant_map : dictionary
            A key value pair mapping from SymPy symbols to floating point
            values.

        Returns
        =======
        track : VectorKeyframeTrack
            PyThreeJS animation track.

        """
        # TODO : Passing in constant_values and constant_map is redundant,
        # right?
        if p3js is None:
            raise ImportError('pythreejs must be installed.')

        # NOTE : WebGL doesn't like 64bit so convert to 32 bit.
        times = np.asarray(times, dtype=np.float32)

        self._mesh = self.shape._p3js_mesh(constant_map=constant_map)

        # NOTE : This is required to set the transform matrix directly.
        self._mesh.matrixAutoUpdate = False

        matrices = self.evaluate_transformation_matrix(dynamic_values,
                                                       constant_values)

        # Set initial configuration.
        self._mesh.matrix = matrices[0]

        # TODO : If the user does not name their shapes, then there will be
        # KeyFrameTracks with duplicate names. Need a better fix for this, but
        # I at least warn the user if they didn't change the name at all.
        if self._mesh.name == 'unnamed':
            msg = ("The shape provided to this visualization frame must have a "
                   "unique name other thane 'unnamed'. Make sure all shapes in "
                   "the scene have unique names.")
            raise ValueError(msg)

        name = "scene/{}.matrix".format(self._mesh.name)

        track = p3js.VectorKeyframeTrack(name=name, times=times,
                                         values=matrices)

        self._track = track



#!/usr/bin/env python

import os

import numpy as np
import sympy as sm

from ...models import multi_mass_spring_damper
from ..c_code import _CLUsolveGenerator
from ..cython_code import CythonMatrixGenerator


class TestCythonMatrixGenerator(object):

    def setup(self):

        self.prefix = 'boogly_bee'

        sys = multi_mass_spring_damper(6, True, True)

        self.matrices = (sys.eom_method.mass_matrix,
                         sys.eom_method.forcing)

        self.arguments = (sys.constants_symbols,
                          sys.coordinates,
                          sys.speeds,
                          sys.specifieds_symbols)

        self.generator = CythonMatrixGenerator(self.arguments,
                                               self.matrices, self.prefix)

    def test_generate_code_blocks(self):

        expected = {}

        expected['header_args'] = \
"""\
                  double* input_0,
                  double* input_1,
                  double* input_2,
                  double* input_3,
                  double* output_0,
                  double* output_1\
"""

        expected['python_args'] = \
"""\
         np.ndarray[np.double_t, ndim=1, mode='c'] input_0,
         np.ndarray[np.double_t, ndim=1, mode='c'] input_1,
         np.ndarray[np.double_t, ndim=1, mode='c'] input_2,
         np.ndarray[np.double_t, ndim=1, mode='c'] input_3,
         np.ndarray[np.double_t, ndim=1, mode='c'] output_0,
         np.ndarray[np.double_t, ndim=1, mode='c'] output_1\
"""

        expected['c_args'] = \
"""\
             <double*> input_0.data,
             <double*> input_1.data,
             <double*> input_2.data,
             <double*> input_3.data,
             <double*> output_0.data,
             <double*> output_1.data\
"""

        expected['output'] = \
"""\
            output_0.reshape(6, 6),
            output_1\
"""

        self.generator._generate_code_blocks()

        for k, v in self.generator.code_blocks.items():
            assert v == expected[k]

    def test_doprint(self):

        expected_pyx_source = \
"""\
import numpy as np
cimport numpy as np
cimport cython

cdef extern from "boogly_bee_c.h":
    void evaluate(
                  double* input_0,
                  double* input_1,
                  double* input_2,
                  double* input_3,
                  double* output_0,
                  double* output_1
                 )

@cython.boundscheck(False)
@cython.wraparound(False)
def eval(
         np.ndarray[np.double_t, ndim=1, mode='c'] input_0,
         np.ndarray[np.double_t, ndim=1, mode='c'] input_1,
         np.ndarray[np.double_t, ndim=1, mode='c'] input_2,
         np.ndarray[np.double_t, ndim=1, mode='c'] input_3,
         np.ndarray[np.double_t, ndim=1, mode='c'] output_0,
         np.ndarray[np.double_t, ndim=1, mode='c'] output_1
        ):

    evaluate(
             <double*> input_0.data,
             <double*> input_1.data,
             <double*> input_2.data,
             <double*> input_3.data,
             <double*> output_0.data,
             <double*> output_1.data
            )

    return (
            output_0.reshape(6, 6),
            output_1
           )\
"""

        expected_setup_py_source = """\
#!/usr/bin/env python

from setuptools import setup
from setuptools import Extension

from Cython.Build import cythonize
import numpy

extension = Extension(name="boogly_bee",
                      sources=["boogly_bee.pyx",
                               "boogly_bee_c.c"],
                      include_dirs=[numpy.get_include()])

setup(name="boogly_bee",
      ext_modules=cythonize([extension]))\
"""

        setup, pyx, c_header, c_source = self.generator.doprint()

        assert setup == expected_setup_py_source
        assert pyx == expected_pyx_source

    def test_write(self):

        setup, pyx, c_header, c_source = self.generator.doprint()

        self.generator.write()

        with open(self.prefix + '_c.h') as f:
            assert f.read() == c_header

        with open(self.prefix + '_c.c') as f:
            assert f.read() == c_source

        with open(self.prefix + '_setup.py') as f:
            assert f.read() == setup

        with open(self.prefix + '.pyx') as f:
            assert f.read() == pyx

    def test_compile(self):

        f = self.generator.compile()

        subs = {}

        args = []
        for argset in self.arguments:
            vals = np.random.random(len(argset))
            args.append(vals)
            for arg, val in zip(argset, vals):
                subs[arg] = val

        for matrix in self.matrices:
            nr, nc = matrix.shape
            args.append(np.empty(nr * nc, dtype=float))

        for output, expected in zip(f(*args), self.matrices):
            try:
                expected = sm.matrix2numpy(expected.subs(subs),
                                           dtype=float).squeeze()
            except TypeError:
                # dtype kwarg in not supported in earlier SymPy versions
                expected = np.asarray(sm.matrix2numpy(expected.subs(subs)),
                                      dtype=float).squeeze()

            np.testing.assert_allclose(output, expected)

    def teardown(self):

        for suffix in ['_c.h', '_c.c', '_setup.py', '.pyx']:
            filename = self.prefix + suffix
            if os.path.isfile(filename):
                os.remove(filename)


def test_lusolve_generator():
    """Tests whether the symbolic LUsolve of the cse'd expressions results in
    the same answer."""

    sys = multi_mass_spring_damper(6, True, True)

    arguments = (sys.constants_symbols, sys.coordinates, sys.speeds,
                 sys.specifieds_symbols)

    # first two outputs have to be A and b of Ax=b
    outputs = [sys.eom_method.mass_matrix,  # A
               sys.eom_method.forcing,  # b
               sys.eom_method.forcing]

    generator = CythonMatrixGenerator(arguments, outputs)
    # patch in the special generator
    generator.c_matrix_generator = _CLUsolveGenerator(arguments, outputs)
    func = generator.compile()

    # setup the input and output arrays
    args = []
    subs = {}
    for argset in arguments:
        vals = np.random.random(len(argset))
        args.append(vals)
        for arg, val in zip(argset, vals):
            subs[arg] = val

    nr, nc = sys.eom_method.mass_matrix.shape
    A_vals = np.empty(nr*nc, dtype=float)
    x_vals = np.empty(nr, dtype=float)
    b_vals = np.empty(nr, dtype=float)

    func(*(args + [A_vals, x_vals, b_vals]))

    gen2 = CythonMatrixGenerator(arguments, [sys.eom_method.mass_matrix,
                                             sys.eom_method.forcing])
    func2 = gen2.compile()
    M_vals = np.empty(nr*nc, dtype=float)
    F_vals = np.empty(nr, dtype=float)
    func2(*(args + [M_vals, F_vals]))

    np.testing.assert_allclose(M_vals, A_vals)
    np.testing.assert_allclose(F_vals, b_vals)
    np.testing.assert_allclose(x_vals,
                               np.linalg.solve(M_vals.reshape((nr, nc)),
                                               F_vals))



#!/usr/bin/env python

import os
import sys
import shutil
import tempfile
import importlib
import subprocess
from collections import defaultdict

from .c_code import CMatrixGenerator
from ..utils import wrap_and_indent


class CythonMatrixGenerator(object):
    """This class generates the Cython code for evaluating a sequence of
    matrices. It can compile the code and return a Python function."""

    _pyx_template = \
"""\
import numpy as np
cimport numpy as np
cimport cython

cdef extern from "{prefix}_c.h":
    void evaluate(
{header_args}
                 )

@cython.boundscheck(False)
@cython.wraparound(False)
def eval(
{python_args}
        ):

    evaluate(
{c_args}
            )

    return (
{output}
           )\
"""

    _setup_py_template = """\
#!/usr/bin/env python

from setuptools import setup
from setuptools import Extension

from Cython.Build import cythonize
import numpy

extension = Extension(name="{prefix}",
                      sources=["{prefix}.pyx",
                               "{prefix}_c.c"],
                      include_dirs=[numpy.get_include()])

setup(name="{prefix}",
      ext_modules=cythonize([extension]))\
"""

    _module_counter = 0

    def __init__(self, arguments, matrices, prefix='pydy_codegen', cse=True):
        """

        Parameters
        ==========
        arguments : sequences of sequences of SymPy Symbol or Function.
            Each of the sequences will be converted to input arrays in the
            Cython function. All of the symbols/functions contained in
            ``matrices`` need to be in the sequences, but the sequences can
            also contain extra symbols/functions that are not contained in
            the matrices.
        matrices : sequence of SymPy.Matrix
            A sequence of the matrices that should be evaluated in the
            function. The expressions should contain only sympy.Symbol or
            sympy.Function that are functions of me.dynamicsymbols._t.
        prefix : string, optional
            The desired prefix for the generated files.
        cse : boolean
            Find and replace common sub-expressions in ``matrices`` if True.

        """

        self.prefix = prefix
        self.matrices = matrices
        self.arguments = arguments
        self.num_matrices = len(matrices)
        self.num_arguments = len(arguments)
        self.c_matrix_generator = CMatrixGenerator(arguments, matrices,
                                                   cse=cse)

        self._generate_code_blocks()

    def _generate_code_blocks(self):

        lines = defaultdict(list)

        hd = 'double* {}_{},'
        py = "np.ndarray[np.double_t, ndim=1, mode='c'] {}_{},"
        c = '<double*> {}_{}.data,'
        out = 'output_{}.reshape({}, {}),'
        out_vec = 'output_{},'

        for i in range(self.num_arguments):
            lines['header_args'].append(hd.format('input', i))
            lines['python_args'].append(py.format('input', i))
            lines['c_args'].append(c.format('input', i))

        for i, matrix in enumerate(self.matrices):
            lines['header_args'].append(hd.format('output', i))
            lines['python_args'].append(py.format('output', i))
            lines['c_args'].append(c.format('output', i))
            nr, nc = matrix.shape
            if nc == 1:
                lines['output'].append(out_vec.format(i))
            else:
                lines['output'].append(out.format(i, nr, nc))

        indents = {'header_args': 18,
                   'python_args': 9,
                   'c_args': 13,
                   'output': 12}

        self.code_blocks = {k: wrap_and_indent(v, indents[k])[:-1] for k, v
                            in lines.items()}

    def doprint(self):
        """Returns the text of the four source files needed to compile
        Cython wrapper that evaluates the provided SymPy matrices.

        Returns
        =======
        setup_py : string
            The text of the setup.py file used to compile the Cython
            extension.
        cython_source : string
            The text of the Cython pyx file which includes the wrapper
            function ``eval``.
        c_header : string
            The text of the C header file that exposes the evaluate
            function.
        c_source : string
            The text of the C source file containing the function that
            evaluates the matrices.

        """

        c_header, c_source = self.c_matrix_generator.doprint(
            prefix=self.prefix + '_c')

        filling = {'prefix': self.prefix}
        filling.update(self.code_blocks)

        cython_source = self._pyx_template.format(**filling)
        setup_py = self._setup_py_template.format(**filling)

        return setup_py, cython_source, c_header, c_source

    def write(self, path=None):
        """Writes the four source files needed to compile the Cython
        function to the current working directory.

        Parameters
        ==========
        path : string
            The absolute or relative path to an existing directory to place
            the files instead of the cwd.

        """

        if path is None:
            path = os.getcwd()

        self.c_matrix_generator.write(self.prefix + '_c')

        setup_py, pyx, c_header, c_source = self.doprint()

        with open(os.path.join(path, self.prefix + '_setup.py'), 'w') as f:
            f.write(setup_py)

        with open(os.path.join(path, self.prefix + '.pyx'), 'w') as f:
            f.write(pyx)

    def compile(self, tmp_dir=None, verbose=False):
        """Returns a function which evaluates the matrices.

        Parameters
        ==========
        tmp_dir : string
            The path to an existing or non-existing directory where all of
            the generated files will be stored.
        verbose : boolean
            If true the output of the completed compilation steps will be
            printed.

        """

        base_prefix = self.prefix

        if tmp_dir is None:
            codedir = tempfile.mkdtemp(".pydy_compile")
        else:
            codedir = os.path.abspath(tmp_dir)

        if not os.path.exists(codedir):
            os.makedirs(codedir)

        self.prefix = '{}_{}'.format(base_prefix,
                                     CythonMatrixGenerator._module_counter)

        workingdir = os.getcwd()
        os.chdir(codedir)

        try:
            sys.path.append(codedir)
            self.write()
            cmd = [sys.executable, self.prefix + '_setup.py', 'build_ext',
                   '--inplace']
            output = subprocess.check_output(cmd, stderr=subprocess.STDOUT)
            if verbose:
                print(output.decode())
            cython_module = importlib.import_module(self.prefix)
        except:
            raise Exception('Failed to compile and import Cython module.')
        finally:
            sys.path.remove(codedir)
            CythonMatrixGenerator._module_counter += 1
            os.chdir(workingdir)
            if tmp_dir is None:
                # rmtree fails on Windows with permissions errors, so skip the
                # removal on Windows.
                try:
                    shutil.rmtree(codedir)
                except OSError:
                    pass

        self.prefix = base_prefix

        return getattr(cython_module, 'eval')



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# external libraries
import sympy as sm
import sympy.physics.mechanics as me

# local
from ..models import multi_mass_spring_damper


def test_multi_mass_spring_damper():

    k0, m0, g, c0 = sm.symbols('k0, m0, g, c0')
    x0, v0, f0 = me.dynamicsymbols('x0, v0, f0')

    sys = multi_mass_spring_damper()

    assert sys.constants_symbols == {k0, c0, m0}
    assert sys.specifieds_symbols == set()
    assert sys.coordinates == [x0]
    assert sys.speeds == [v0]
    assert sys.states == [x0, v0]

    expected_mass_matrix_full = sm.Matrix([[1, 0], [0, m0]])
    expected_forcing_full = sm.Matrix([[v0],
                                       [-c0 * v0 - k0 * x0]])

    assert sm.simplify(sys.eom_method.mass_matrix_full -
                       expected_mass_matrix_full) == sm.zeros(2, 2)
    assert sm.simplify(sys.eom_method.forcing_full -
                       expected_forcing_full) == sm.zeros(2, 1)


def test_multi_mass_spring_damper_with_forces():

    k0, m0, g, c0 = sm.symbols('k0, m0, g, c0')
    x0, v0, f0 = me.dynamicsymbols('x0, v0, f0')

    sys = multi_mass_spring_damper(apply_gravity=True,
                                   apply_external_forces=True)

    assert sys.constants_symbols == {k0, m0, g, c0}
    assert sys.specifieds_symbols == {f0}
    assert sys.coordinates == [x0]
    assert sys.speeds == [v0]
    assert sys.states == [x0, v0]

    expected_mass_matrix_full = sm.Matrix([[1, 0], [0, m0]])
    expected_forcing_full = sm.Matrix([[v0],
                                       [-c0 * v0 + g * m0 - k0 * x0 + f0]])

    assert sm.simplify(sys.eom_method.mass_matrix_full -
                       expected_mass_matrix_full) == sm.zeros(2, 2)
    assert sm.simplify(sys.eom_method.forcing_full -
                       expected_forcing_full) == sm.zeros(2, 1)


def test_multi_mass_spring_damper_double():

    m0, m1, c0, c1, k0, k1, g = sm.symbols('m0, m1, c0, c1, k0, k1, g')
    x0, x1, v0, v1, f0, f1 = me.dynamicsymbols('x0, x1, v0, v1, f0, f1')

    sys = multi_mass_spring_damper(2, True, True)

    assert sys.constants_symbols == \
        {c1, m1, k0, c0, k1, g, m0}
    assert sys.specifieds_symbols == {f0, f1}
    assert sys.coordinates == [x0, x1]
    assert sys.speeds == [v0, v1]
    assert sys.states == [x0, x1, v0, v1]

    expected_mass_matrix_full = sm.Matrix([[1, 0,       0,  0],
                                           [0, 1,       0,  0],
                                           [0, 0, m0 + m1, m1],
                                           [0, 0,      m1, m1]])
    expected_forcing_full = sm.Matrix([[v0],
                                       [v1],
                                       [-c0 * v0 + g * m0 + g * m1 - k0 * x0 + f0 + f1],
                                       [-c1 * v1 + g * m1 - k1 * x1 + f1]])

    assert sm.simplify(sys.eom_method.mass_matrix_full -
                       expected_mass_matrix_full) == sm.zeros(4, 4)
    assert sm.simplify(sys.eom_method.forcing_full -
                       expected_forcing_full) == sm.zeros(4, 1)



#!/usr/bin/env python

import sys
if sys.version_info > (3, 0):
    from collections.abc import Sequence
else:
    from collections import Sequence
from itertools import chain

import numpy as np
import numpy.linalg
import scipy.linalg
import sympy as sm
import sympy.physics.mechanics as me
from sympy.core.function import UndefinedFunction, Derivative
Cython = sm.external.import_module('Cython')
theano = sm.external.import_module('theano')
if theano:
    from sympy.printing.theanocode import theano_function

from .c_code import _CLUsolveGenerator
from .cython_code import CythonMatrixGenerator


class ODEFunctionGenerator(object):
    """This is an abstract base class for all of the generators. A subclass
    is expected to implement the methods necessary to evaluate the arrays
    needed to compute xdot for the three different system specification
    types."""

    _rhs_doc_template = \
"""\
Returns the derivatives of the states, i.e. numerically evaluates the right
hand side of the first order differential equation.

x' = f(x, t,{specified_call_sig} p)

Parameters
==========
x : ndarray, shape({num_states},)
    The state vector is ordered as such:
{state_list}
t : float
    The current time.{specifieds_explanation}{constants_explanation}

Returns
=======
dx : ndarray, shape({num_states},)
    The derivative of the state vector.

"""

    _constants_doc_templates = {}

    _constants_doc_templates[None] = \
"""
p : dictionary len({num_constants}) or ndarray shape({num_constants},)
    Either a dictionary that maps the constants symbols to their numerical
    values or an array with the constants in the following order:
{constant_list}\
"""

    _constants_doc_templates['array'] = \
"""
p : ndarray shape({num_constants},)
    A ndarray of floats that give the numerical values of the constants in
    this order:
{constant_list}\
"""

    _constants_doc_templates['dictionary'] = \
"""
p : dictionary len({num_constants})
    A dictionary that maps the constants symbols to their numerical values
    with at least these keys:
{constant_list}\
"""

    _specifieds_doc_templates = {}

    _specifieds_doc_templates[None] = \
"""
r : dictionary; ndarray, shape({num_specified},); function

    There are three options for this argument. (1) is more flexible but
    (2) and (3) are much more efficient.

    (1) A dictionary that maps the specified functions of time to floats,
    ndarrays, or functions that produce ndarrays. The keys can be a single
    specified symbolic function of time or a tuple of symbols. The total
    number of symbols must be equal to {num_specified}. If the value is a
    function it must be of the form g(x, t), where x is the current state
    vector ndarray and t is the current time float and it must return an
    ndarray of the correct shape. For example::

      r = {{a: 1.0,
           (d, b) : np.array([1.0, 2.0]),
           (e, f) : lambda x, t: np.array(x[0], x[1]),
           c: lambda x, t: np.array(x[2])}}

    (2) A ndarray with the specified values in the correct order and of the
    correct shape.

    (3) A function that must be of the form g(x, t), where x is the current
    state vector and t is the current time and it must return an ndarray of
    the correct shape.

    The specified inputs are, in order:
{specified_list}\
"""

    _specifieds_doc_templates['array'] = \
"""
r : ndarray, shape({num_specified},)

    A ndarray with the specified values in the correct order and of the
    correct shape.

    The specified inputs are, in order:
{specified_list}\
"""

    _specifieds_doc_templates['function'] = \
"""
r : function

    A function that must be of the form g(x, t), where x is the current
    state vector and t is the current time and it must return an ndarray of
    shape({num_specified},).

    The specified inputs are, in order:
{specified_list}\
"""

    _specifieds_doc_templates['dictionary'] = \
"""
r : dictionary

    A dictionary that maps the specified functions of time to floats,
    ndarrays, or functions that produce ndarrays. The keys can be a single
    specified symbolic function of time or a tuple of symbols. The total
    number of symbols must be equal to {num_specified}. If the value is a
    function it must be of the form g(x, t), where x is the current state
    vector ndarray and t is the current time float and it must return an
    ndarray of the correct shape. For example::

      r = {{a: 1.0,
           (d, b) : np.array([1.0, 2.0]),
           (e, f) : lambda x, t: np.array(x[0], x[1]),
           c: lambda x, t: np.array(x[2])}}

    The specified inputs are, in order:
{specified_list}\
"""

    @staticmethod
    def _deduce_system_type(**kwargs):
        """Based on the combination of arguments this returns which ODE
        description has been provided.

        full rhs
            x' = f(x, t, r, p)
        full mass matrix
            M(x, p) * x' = f(x, t, r, p)
        min mass matrix
            M(q, p) * u' = f(q, u, t, r, p)
            q' = g(q, u, t)

        """

        if kwargs.pop('coordinate_derivatives') is not None:
            system_type = 'min mass matrix'
        elif kwargs.pop('mass_matrix') is not None:
            system_type = 'full mass matrix'
        else:
            system_type = 'full rhs'

        return system_type

    def __init__(self, right_hand_side, coordinates, speeds, constants=(),
                 mass_matrix=None, coordinate_derivatives=None,
                 specifieds=None, linear_sys_solver='numpy',
                 constants_arg_type=None, specifieds_arg_type=None):
        """Generates a numerical function which can evaluate the right hand
        side of the first order ordinary differential equations from a
        system described by one of the following three symbolic forms:

            [1] x' = F(x, t, r, p)

            [2] M(x, p) x' = F(x, t, r, p)

            [3] M(q, p) u' = F(q, u, t, r, p)
                q' = G(q, u, t, r, p)

        where

            x : states, i.e. [q, u]
            t : time
            r : specified (exogenous) inputs
            p : constants
            q : generalized coordinates
            u : generalized speeds
            M : mass matrix (full or minimum)
            F : right hand side (full or minimum)
            G : right hand side of the kinematical differential equations

        The generated function is of the form F(x, t, p) or F(x, t, r, p)
        depending on whether the system has specified inputs or not.

        Parameters
        ==========
        right_hand_side : SymPy Matrix, shape(n, 1)
            A column vector containing the symbolic expressions for the
            right hand side of the ordinary differential equations. If the
            right hand side has been solved for symbolically then only F is
            required, see form [1]; if not then the mass matrix must also be
            supplied, see forms [2, 3].
        coordinates : sequence of SymPy Functions
            The generalized coordinates. These must be ordered in the same
            order as the rows in M, F, and/or G and be functions of time.
        speeds : sequence of SymPy Functions
            The generalized speeds. These must be ordered in the same order
            as the rows in M, F, and/or G and be functions of time.
        constants : sequence of SymPy Symbols, optional
            All of the constants present in the equations of motion. The
            order does not matter.
        mass_matrix : sympy.Matrix, shape(n, n), optional
            This can be either the "full" mass matrix as in [2] or the
            "minimal" mass matrix as in [3]. The rows and columns must be
            ordered to match the order of the coordinates and speeds. In the
            case of the full mass matrix, the speeds should always be
            ordered before the speeds, i.e. x = [q, u].
        coordinate_derivatives : sympy.Matrix, shape(m, 1), optional
            If the "minimal" mass matrix, form [3], is supplied, then this
            column vector represents the right hand side of the kinematical
            differential equations.
        specifieds : sequence of SymPy Functions
            The specified exogenous inputs to the system. These should be
            functions of time and the order does not matter.
        linear_sys_solver : string or function
            Specify either `numpy` or `scipy` to use the linear solvers
            provided in each package or supply a function that solves a
            linear system Ax=b with the call signature x = solve(A, b). For
            example, if you need to use custom kwargs for the SciPy solver,
            pass in a lambda function that wraps the solver and sets them.
        constants_arg_type : string
            The generated function accepts two different types of arguments
            for the numerical values of the constants: either a ndarray of
            the constants values in the correct order or a dictionary
            mapping the constants symbols to the numerical values. If None,
            this is determined inside of the generated function and can
            cause a significant slow down for performance critical code. If
            you know apriori what arg types you need to support choose
            either ``array`` or ``dictionary``. Note that ``array`` is
            faster than ``dictionary``.
        specifieds_arg_type : string
            The generated function accepts three different types of
            arguments for the numerical values of the specifieds: either a
            ndarray of the specifieds values in the correct order, a
            function that generates the correctly ordered ndarray, or a
            dictionary mapping the specifieds symbols or tuples of thereof
            to floats, ndarrays, or functions. If None, this is determined
            inside of the generated function and can cause a significant
            slow down for performance critical code. If you know apriori
            what arg types you want to support choose either ``array``,
            ``function``, or ``dictionary``. The speed of each, from fast to
            slow, are ``array``, ``function``, ``dictionary``, None.

        """

        self.right_hand_side = right_hand_side
        self.coordinates = coordinates
        self.speeds = speeds
        self.constants = constants
        self.mass_matrix = mass_matrix
        self.coordinate_derivatives = coordinate_derivatives
        self.specifieds = specifieds
        self.linear_sys_solver = linear_sys_solver
        self.constants_arg_type = constants_arg_type
        self.specifieds_arg_type = specifieds_arg_type

        # As the order of the constants and specifieds arguments if not
        # important, allow Sets to be used as input. However, the order must be
        # maintained and converted to a Sequence.
        if constants is not None and not isinstance(constants, Sequence):
            self.constants = tuple(constants)
        if specifieds is not None and not isinstance(specifieds, Sequence):
            self.specifieds = tuple(specifieds)

        self.system_type = self._deduce_system_type(
            mass_matrix=mass_matrix,
            coordinate_derivatives=coordinate_derivatives)

        self.num_coordinates = len(coordinates)
        self.num_speeds = len(speeds)
        self.num_states = self.num_coordinates + self.num_speeds
        self.num_constants = len(constants)

        if self.specifieds is None:
            self.num_specifieds = 0
            self.specifieds_arg_type = None
        else:
            self.num_specifieds = len(specifieds)

        # These are pre-allocated storage for the numerical values used in
        # some of the rhs() evaluations.
        self._constants_values = np.empty(self.num_constants)
        self._specifieds_values = np.empty(self.num_specifieds)

        self._check_system_consitency()

    @property
    def linear_sys_solver(self):
        return self._linear_sys_solver

    @linear_sys_solver.setter
    def linear_sys_solver(self, v):

        if isinstance(v, type(lambda x: x)):
            self._solve_linear_system = v
            self._linear_sys_solver = v
        elif v == 'numpy':
            self._solve_linear_system = numpy.linalg.solve
            self._linear_sys_solver = v
        elif v == 'scipy':
            self._solve_linear_system = scipy.linalg.solve
            self._linear_sys_solver = v
        elif v == 'sympy':
            # dummy function
            self._solve_linear_system = lambda A, b: np.nan*np.ones_like(b)
            self._linear_sys_solver = v
        else:
            msg = '{} is not a valid solver.'
            raise ValueError(msg.format(self.linear_sys_solver))

    def _check_system_consitency(self):

        if self.system_type == 'min mass matrix':

            nr, nc = self.mass_matrix.shape
            assert self.num_speeds == nr == nc
            assert self.num_speeds == self.right_hand_side.shape[0]
            assert self.num_coordinates == self.coordinate_derivatives.shape[0]

        elif self.system_type == 'full mass matrix':

            nr, nc = self.mass_matrix.shape
            assert self.num_states == nr == nc
            assert self.num_states == self.right_hand_side.shape[0]
            assert self.coordinate_derivatives is None

        elif self.system_type == 'full rhs':

            assert self.num_states == self.right_hand_side.shape[0]
            assert self.mass_matrix is None
            assert self.coordinate_derivatives is None

    @staticmethod
    def list_syms(indent, syms):
        """Returns a string representation of a valid rst list of the
        symbols in the sequence syms and indents the list given the integer
        number of indentations."""
        indentation = ' ' * indent
        lst = '- ' + ('\n' + indentation + '- ').join([str(s) for s in syms])
        return indentation + lst

    def _convert_constants_dict_to_array(self, p):
        """Returns an array of numerical values from the constants
        dictionary in the correct order."""

        # NOTE : It's unfortunate that this has to be run at every rhs eval,
        # because subsequent calls to rhs() doesn't require different
        # constants. I suppose you can sub out all the constants in the EoMs
        # before passing them into the generator. That would beg for the
        # capability to support self.constants=None to skip all of this
        # stuff in the rhs eval.
        for i, c in enumerate(self.constants):
            self._constants_values[i] = p[c]

        return self._constants_values

    def _parse_constants(self, *args):
        """Returns an ndarray containing the numerical values of the
        constants in the correct order. If the constants are already an
        array, that array is returned."""

        p = args[-1]
        try:
            # NOTE : This emits "VisibleDeprecationWarning: using a non-integer
            # number instead of an integer will result in an error in the
            # future" along with the IndexError in NumPy 1.11. Not sure why it
            # gives the warning, it already gives and error with trying to
            # index with a SymPy symbol.
            p = self._convert_constants_dict_to_array(p)
        except IndexError:
            # p is an array so just return the args
            return args
        else:
            return args[:-1] + (p,)

    def _convert_specifieds_dict_to_array(self, x, t, r):

        for k, v in r.items():
            # TODO : Not sure if this is the best check here.
            if (isinstance(type(k), UndefinedFunction) or
                isinstance(k, Derivative)):
                k = (k,)
            idx = [self.specifieds.index(symmy) for symmy in k]
            try:
                self._specifieds_values[idx] = v(x, t)
            except TypeError:  # not callable
                # If not callable, then it should be a float, ndarray,
                # or indexable.
                self._specifieds_values[idx] = v

        return self._specifieds_values

    def _parse_specifieds(self, x, t, r, p):

        if isinstance(r, dict):
            # NOTE : This function sets self._specifieds_values, so here we
            # return nothing.
            self._convert_specifieds_dict_to_array(x, t, r)
        else:
            # More efficient.
            try:
                self._specifieds_values = r(x, t)
            except TypeError:  # not callable.
                # If not callable, then it should be a float or ndarray.
                self._specifieds_values = r

        return x, t, self._specifieds_values, p

    def _parse_all_args(self, *args):
        """Returns args formatted for the post 0.3.0 generators using all of
        the parsers. This is the slowest method and is used by default if no
        information is provided by the user on which type of args will be
        passed in."""

        args = self._parse_constants(*args)

        if self.specifieds is not None:
            args = self._parse_specifieds(*args)

        return args

    def _generate_rhs_docstring(self):

        template_values = {'num_states': self.num_states,
                           'state_list': self.list_syms(8, self.coordinates
                                                        + self.speeds),
                           'specified_call_sig': '',
                           'constants_explanation':
                               self._constants_doc_templates[
                                   self.constants_arg_type].format(**{
                                       'num_constants': self.num_constants,
                                       'constant_list': self.list_syms(
                                           8, self.constants)}),
                           'specifieds_explanation': ''}

        if self.specifieds is not None:
            template_values['specified_call_sig'] = ' r,'
            specified_template_values = {
                'num_specified': self.num_specifieds,
                'specified_list': self.list_syms(8, self.specifieds)}
            template_values['specifieds_explanation'] = \
                self._specifieds_doc_templates[self.specifieds_arg_type].format(
                    **specified_template_values)

        return self._rhs_doc_template.format(**template_values)

    def _create_rhs_function(self):
        """Returns a function in the form expected by scipy.integrate.odeint
        that computes the derivatives of the states."""

        p_arg_type = self.constants_arg_type
        r_arg_type = self.specifieds_arg_type

        if p_arg_type is None and r_arg_type is None:
            def rhs(*args):
                # args: x, t, p
                # or
                # args: x, t, r, p

                args = self._parse_all_args(*args)

                q = args[0][:self.num_coordinates]
                u = args[0][self.num_coordinates:]

                if self.constants:
                    xdot = self._base_rhs(q, u, *args[2:])
                else:
                    xdot = self._base_rhs(q, u, *(args[2:3] + ([],)))
                return xdot

            rhs.__doc__ = self._generate_rhs_docstring()

            return rhs

        if p_arg_type == 'dictionary':
            p = lambda *li : self._convert_constants_dict_to_array(li[-1])

        else:
            p = lambda *li : self._parse_constants(*li)[-1]

        if r_arg_type is None:
            if self.specifieds is not None:
                r = lambda *li : self._parse_specifieds(*li)[-2]

        elif r_arg_type == 'array':
            r = lambda *li : (li)[-2]

        elif r_arg_type == 'dictionary':
            r = lambda *li : self._convert_specifieds_dict_to_array(*li[:3])

        elif r_arg_type == 'function':
            r = lambda *li: li[2](*li[2:])

        def rhs(*args):
            # args: x, t, p
            # or
            # args: x, t, r, p

            q = args[0][:self.num_coordinates]
            u = args[0][self.num_coordinates:]

            if self.specifieds is None:
                if self.constants:
                    return self._base_rhs(q, u, p(*args))
                else:
                    return self._base_rhs(q, u, [])
            else:
                if self.constants:
                    return self._base_rhs(q, u, r(*args), p(*args))
                else:
                    return self._base_rhs(q, u, r(*args), [])

        rhs.__doc__ = self._generate_rhs_docstring()

        return rhs

    def _create_base_rhs_function(self):
        """Sets the self._base_rhs function. This function accepts arguments
        in this form: (q, u, p) or (q, u, r, p)."""

        if self.system_type == 'full rhs':

            self._base_rhs = self.eval_arrays

        elif (self.system_type == 'full mass matrix' and
              self.linear_sys_solver=='sympy'):

            def base_rhs(*args):
                M, xdot = self.eval_arrays(*args)
                return xdot

            self._base_rhs = base_rhs

        elif self.system_type == 'full mass matrix':

            def base_rhs(*args):

                M, F = self.eval_arrays(*args)
                return self._solve_linear_system(M, F)

            self._base_rhs = base_rhs

        elif (self.system_type == 'min mass matrix' and
              self.linear_sys_solver=='sympy'):

            xdot = np.empty(self.num_states, dtype=float)

            def base_rhs(*args):
                M, udot, qdot = self.eval_arrays(*args)
                xdot[:self.num_coordinates] = qdot
                xdot[self.num_coordinates:] = udot
                return xdot

            self._base_rhs = base_rhs

        elif self.system_type == 'min mass matrix':

            xdot = np.empty(self.num_states, dtype=float)

            def base_rhs(*args):
                M, F, qdot = self.eval_arrays(*args)
                if self.num_speeds == 1:
                    udot = F / M
                else:
                    udot = self._solve_linear_system(M, F)
                xdot[:self.num_coordinates] = qdot
                xdot[self.num_coordinates:] = udot
                return xdot

            self._base_rhs = base_rhs

    def define_inputs(self):
        """Sets self.inputs to the list of sequences [q, u, p] or [q, u, r,
        p]."""

        self.inputs = [self.coordinates, self.speeds, self.constants]
        if self.specifieds is not None:
            self.inputs.insert(2, self.specifieds)

    def generate(self):
        """Returns a function that evaluates the right hand side of the
        first order ordinary differential equations in one of two forms:

            x' = f(x, t, p)

            or

            x' = f(x, t, r, p)

        See the docstring of the generated function for more details.

        """

        if self.system_type == 'full rhs':
            self.generate_full_rhs_function()
        elif self.system_type == 'full mass matrix':
            self.generate_full_mass_matrix_function()
        elif self.system_type == 'min mass matrix':
            self.generate_min_mass_matrix_function()

        self._create_base_rhs_function()

        return self._create_rhs_function()


class CythonODEFunctionGenerator(ODEFunctionGenerator):

    def __init__(self, *args, **kwargs):

        self._options = {'tmp_dir': None,
                         'prefix': 'pydy_codegen',
                         'cse': True,
                         'verbose': False,
                         }
        for k, v in self._options.items():
            self._options[k] = kwargs.pop(k, v)

        if Cython is None:
            raise ImportError('Cython must be installed to use this class.')
        else:
            super(CythonODEFunctionGenerator, self).__init__(*args, **kwargs)

    __init__.__doc__ = ODEFunctionGenerator.__init__.__doc__

    def _cythonize(self, outputs, inputs):
        g = CythonMatrixGenerator(inputs, outputs,
                                  prefix=self._options['prefix'],
                                  cse=self._options['cse'])
        return g.compile(tmp_dir=self._options['tmp_dir'],
                         verbose=self._options['verbose'])

    def _cythonize_symbolic_lusolve(self, outputs, inputs):
        if not self._options['cse']:
            msg = 'cse has to be True if using the sympy linear system solver'
            raise ValueError(msg)
        g = CythonMatrixGenerator(inputs, outputs,
                                  prefix=self._options['prefix'],
                                  cse=True)
        # patch in the special generator
        g.c_matrix_generator = _CLUsolveGenerator(inputs, outputs)
        return g.compile(tmp_dir=self._options['tmp_dir'],
                         verbose=self._options['verbose'])

    def _set_eval_array(self, f):

        if self.specifieds is None:
            self.eval_arrays = lambda q, u, p: f(q, u, p, *self._empties)
        else:
            self.eval_arrays = lambda q, u, r, p: f(q, u, r, p,
                                                    *self._empties)

    def generate_full_rhs_function(self):

        self.define_inputs()
        outputs = [self.right_hand_side]

        self._empties = (np.empty(self.num_states, dtype=float),)

        self._set_eval_array(self._cythonize(outputs, self.inputs))

    def generate_full_mass_matrix_function(self):

        self.define_inputs()
        outputs = [self.mass_matrix, self.right_hand_side]

        mass_matrix_result = np.empty(self.num_states ** 2, dtype=float)
        rhs_result = np.empty(self.num_states, dtype=float)

        self._empties = (mass_matrix_result, rhs_result)

        if self.linear_sys_solver == 'sympy':
            self._set_eval_array(self._cythonize_symbolic_lusolve(outputs,
                                                                  self.inputs))
        else:
            self._set_eval_array(self._cythonize(outputs, self.inputs))

    def generate_min_mass_matrix_function(self):

        self.define_inputs()
        outputs = [self.mass_matrix, self.right_hand_side,
                   self.coordinate_derivatives]

        mass_matrix_result = np.empty(self.num_speeds ** 2, dtype=float)
        rhs_result = np.empty(self.num_speeds, dtype=float)
        kin_diffs_result = np.empty(self.num_coordinates, dtype=float)
        self._empties = (mass_matrix_result, rhs_result, kin_diffs_result)

        if self.linear_sys_solver == 'sympy':
            self._set_eval_array(self._cythonize_symbolic_lusolve(outputs,
                                                                  self.inputs))
        else:
            self._set_eval_array(self._cythonize(outputs, self.inputs))


class LambdifyODEFunctionGenerator(ODEFunctionGenerator):

    def _lambdify(self, outputs):
        # TODO : We could forgo this substitution for generation speed
        # purposes and have lots of args for lambdify (like it used to be
        # done) but there may be some limitations on number of args.
        subs = {}
        vec_inputs = []
        if self.specifieds is None:
            def_vecs = ['q', 'u', 'p']
        else:
            def_vecs = ['q', 'u', 'r', 'p']

        for syms, vec_name in zip(self.inputs, def_vecs):
            v = sm.DeferredVector(vec_name)
            for i, sym in enumerate(syms):
                subs[sym] = v[i]
            vec_inputs.append(v)

        try:
            outputs = [me.msubs(output, subs) for output in outputs]
        except AttributeError:
            # msubs doesn't exist in SymPy < 0.7.6.
            outputs = [output.subs(subs) for output in outputs]

        modules = [{'ImmutableMatrix': np.array}, 'numpy']

        return sm.lambdify(vec_inputs, outputs, modules=modules)

    def generate_full_rhs_function(self):

        self.define_inputs()
        outputs = [self.right_hand_side]

        f = self._lambdify(outputs)

        if self.specifieds is None:
            self.eval_arrays = lambda q, u, p: np.squeeze(f(q, u, p))
        else:
            self.eval_arrays = lambda q, u, r, p: np.squeeze(f(q, u, r, p))

    def generate_full_mass_matrix_function(self):

        self.define_inputs()
        outputs = [self.mass_matrix, self.right_hand_side]

        f = self._lambdify(outputs)

        if self.specifieds is None:
            self.eval_arrays = lambda q, u, p: tuple([np.squeeze(o) for o in
                                                      f(q, u, p)])
        else:
            self.eval_arrays = lambda q, u, r, p: tuple([np.squeeze(o) for o
                                                         in f(q, u, r, p)])

    def generate_min_mass_matrix_function(self):

        self.define_inputs()
        outputs = [self.mass_matrix, self.right_hand_side,
                   self.coordinate_derivatives]

        f = self._lambdify(outputs)

        if self.specifieds is None:
            self.eval_arrays = lambda q, u, p: tuple([np.squeeze(o) for o in
                                                      f(q, u, p)])
        else:
            self.eval_arrays = lambda q, u, r, p: tuple([np.squeeze(o) for o
                                                         in f(q, u, r, p)])


class TheanoODEFunctionGenerator(ODEFunctionGenerator):

    def __init__(self, *args, **kwargs):

        if theano is None:
            raise ImportError('Theano must be installed to use this class.')
        else:
            super(TheanoODEFunctionGenerator, self).__init__(*args, **kwargs)

    __init__.__doc__ = ODEFunctionGenerator.__init__.__doc__

    def define_inputs(self):
        # Theano's input requires a flatted sequence instead of sequence of
        # sequences.
        specifieds = []
        if self.specifieds is not None:
            specifieds = self.specifieds
        self.inputs = chain(self.coordinates, self.speeds,
                            specifieds, self.constants)

    def _theanoize(self, outputs):

        self.define_inputs()

        old_check_input = theano.config.check_input
        old_allow_gc = theano.config.allow_gc
        try:
            # This affects compilation and removes the input check at each step.
            theano.config.check_input = False

            # Disable Theano garbage collection to lower the number of allocations.
            theano.config.allow_gc = False

            f_imp = theano_function(self.inputs, outputs,
                                    on_unused_input='ignore',
                                    mode=theano.Mode(linker='c'))
        finally:
            theano.config.check_input = old_check_input
            theano.config.allow_gc = old_allow_gc

        # While denoting an input as trusted lowers Theano overhead:
        #     f.trust_input = True
        # we can bypass additional overhead with the following function:
        def f(*args):
            for i in range(len(args)):
                f_imp.input_storage[i].storage[0] = args[i]
            f_imp.fn()
            return [f_imp.output_storage[i].data for i in range(len(outputs))]

        return f

    def generate_full_rhs_function(self):

        outputs = [self.right_hand_side]

        f = self._theanoize(outputs)

        def eval_arrays(*args):
            vals = map(np.asarray, np.hstack(args))
            return np.squeeze(f(*vals))

        self.eval_arrays = eval_arrays

    def generate_full_mass_matrix_function(self):

        outputs = [self.mass_matrix, self.right_hand_side]

        f = self._theanoize(outputs)

        def eval_arrays(*args):
            vals = map(np.asarray, np.hstack(args))
            return tuple([np.squeeze(o) for o in f(*vals)])

        self.eval_arrays = eval_arrays

    def generate_min_mass_matrix_function(self):

        outputs = [self.mass_matrix, self.right_hand_side,
                   self.coordinate_derivatives]

        f = self._theanoize(outputs)

        def eval_arrays(*args):
            vals = map(np.asarray, np.hstack(args))
            return tuple([np.squeeze(o) for o in f(*vals)])

        self.eval_arrays = eval_arrays


def generate_ode_function(*args, **kwargs):
    """This is a function wrapper to the above classes. The docstring is
    automatically generated below."""

    generators = {'lambdify': LambdifyODEFunctionGenerator,
                  'cython': CythonODEFunctionGenerator,
                  'theano': TheanoODEFunctionGenerator}

    generator = kwargs.pop('generator', 'lambdify')

    try:
        # See if user passed in a custom class.
        g = generator(*args, **kwargs)
    except TypeError:
        # See if user passed in a string.
        try:
            Generator = generators[generator]
            g = Generator(*args, **kwargs)
        except KeyError:
            msg = '{} is not a valid generator.'.format(generator)
            raise NotImplementedError(msg)
        else:
            return g.generate()
    else:
        return g.generate()


_docstr = ODEFunctionGenerator.__init__.__doc__
_extra_parameters_doc = \
"""\
        generator : string or and ODEFunctionGenerator, optional
            The method used for generating the numeric right hand side. The
            string options are {'lambdify'|'theano'|'cython'} with
            'lambdify' being the default. You can also pass in a custom
            subclass of ODEFunctionGenerator.

        Returns
        =======
        rhs : function
            A function which evaluates the derivaties of the states. See the
            function's docstring for more details after generation.
"""
generate_ode_function.__doc__ = ('' * 4 + _docstr + _extra_parameters_doc)



#!/usr/bin/env python

import itertools

import sympy as sm
import sympy.physics.mechanics as me
from sympy.printing.codeprinter import CodePrinter

from ..utils import wrap_and_indent, find_dynamicsymbols


class MatrixGenerator(object):
    """This abstract base class generates source files that simultaneously
    numerically evaluate any number of SymPy matrices.

    """

    _idx_start = 0
    _idx_delim = "[]"
    _base_printer = CodePrinter
    # TODO : I opened an issue with SymPy to deal with the type declaration,
    # https://github.com/sympy/sympy/issues/10446.
    _type_declar = 'double '  # prepended to variable introductions
    _line_contin = None
    _comment_char = '#'

    def __init__(self, arguments, matrices, cse=True):
        """

        Parameters
        ==========
        arguments : sequence of sequences of SymPy Symbol or Function
            Each of the sequences will be converted to input arrays in the
            generated function. All of the symbols/functions contained in
            ``matrices`` need to be in the sequences, but the sequences can
            also contain extra symbols/functions that are not contained in the
            matrices.
        matrices : sequence of SymPy.Matrix
            A sequence of the matrices that should be evaluated in the
            function. The expressions should contain only sympy.Symbol or
            sympy.Function that are functions of me.dynamicsymbols._t.
        cse : boolean
            Find and replace common sub-expressions in ``matrices`` if True.

        """

        required_args = set()

        for matrix in matrices:
            # TODO : SymPy 0.7.4 does not have Matrix.free_symbols so we
            # manually compute them instead of calling:
            # required_args |= matrix.free_symbols
            required_args |= set().union(*[i.free_symbols for i in matrix])
            required_args |= find_dynamicsymbols(matrix)

        required_args.remove(me.dynamicsymbols._t)

        all_arguments = set(itertools.chain(*arguments))

        for required_arg in required_args:
            if required_arg not in all_arguments:
                msg = "{} is missing from the argument sequences."
                raise ValueError(msg.format(required_arg))

        self.matrices = matrices
        self.arguments = arguments

        if cse:
            self._generate_cse()
        else:
            self._ignore_cse()

        self._generate_code_blocks()

    def _generate_cse(self, prefix='pydy_'):

        # This makes a big long list of every expression in all of the
        # matrices.
        exprs = []
        for matrix in self.matrices:
            exprs += matrix[:]

        # Compute the common subexpresions.
        gen = sm.numbered_symbols(prefix)
        self.subexprs, simplified_exprs = sm.cse(exprs, symbols=gen)

        # Turn the expressions back into matrices of the same type.
        simplified_matrices = []
        idx = 0
        for matrix in self.matrices:
            num_rows, num_cols = matrix.shape
            length = num_rows * num_cols
            m = type(matrix)(simplified_exprs[idx:idx + length])
            simplified_matrices.append(m.reshape(num_rows, num_cols))
            idx += length

        self.simplified_matrices = tuple(simplified_matrices)

    def _ignore_cse(self):
        self.subexprs = []
        self.simplified_matrices = self.matrices

    def _generate_pydy_printer(self):
        """Returns a subclass of a sympy.printing.Printer to print appropriate
        array index calls for all of the symbols in the equations of motion."""

        array_index_map = {}
        for i, arg_set in enumerate(self.arguments):
            for j, var in enumerate(arg_set):
                array_index_map[var] = r'input_{}{}{}{}'.format(
                    i + self._idx_start, self._idx_delim[0],
                    j + self._idx_start, self._idx_delim[1])

        class PyDyCodePrinter(self._base_printer):

            def _print_Function(self, e):
                if e in array_index_map.keys():
                    return array_index_map[e]
                else:
                    return super(PyDyCodePrinter, self)._print_Function(e)

            def _print_Symbol(self, e):
                if e in array_index_map.keys():
                    return array_index_map[e]
                else:
                    return super(PyDyCodePrinter, self)._print_Symbol(e)

        return PyDyCodePrinter

    def comma_lists(self):
        """Returns a string output for each of the sequences of SymPy
        arguments."""

        comma_lists = []

        for i, arg_set in enumerate(self.arguments):
            comma_lists.append(', '.join([str(s) for s in arg_set]))

        return tuple(comma_lists)

    def _generate_code_blocks(self):
        """Writes the blocks of code for the source file."""

        printer = self._generate_pydy_printer()()

        self.code_blocks = {}

        # Creates a string of comma separated inputs.
        lines = []
        for i, input_arg in enumerate(self.arguments):
            lines.append('{}input_{}'.format(self._type_declar,
                                             i + self._idx_start))
        self.code_blocks['input_args'] = ', '.join(lines)

        # Creates a string of comma separated outputs.
        lines = []
        for i, output_arg in enumerate(self.matrices):
            lines.append('{}output_{}'.format(self._type_declar,
                                              i + self._idx_start))
        self.code_blocks['output_args'] = ', '.join(lines)

        lines = []
        for i, (input_arg, explan) in enumerate(zip(self.arguments,
                                                    self.comma_lists())):
            lines.append('{} input_{} : [{}]'.format(self._comment_char, i +
                                                     self._idx_start, explan))
        self.code_blocks['docstring'] = wrap_and_indent(lines, 0,
                                                        comment=self._comment_char)

        lines = []
        for var, expr in self.subexprs:
            lines.append(printer.doprint(expr, var))
        self.code_blocks['subexprs'] = wrap_and_indent(lines,
                                                       continuation=self._line_contin)

        outputs = ''
        for i, output in enumerate(self.simplified_matrices):
            nr, nc = output.shape
            lhs = sm.MatrixSymbol('output_{}'.format(i + self._idx_start), nr, nc)
            try:
                code_str = printer.doprint(output, lhs)
            except AttributeError:
                # The above fails in SymPy 0.7.4.1 because Matrix printing
                # isn't supported.
                code_lines = []
                for j, element in enumerate(output):
                    assignment = 'output_{}[{}]'.format(i, j)
                    code_lines.append(printer.doprint(element, assignment))
                code_str = '\n'.join(code_lines)
            outputs += wrap_and_indent(code_str.split('\n'),
                                       continuation=self._line_contin)
            if i != len(self.simplified_matrices) - 1:
                outputs += '\n\n'  # space between each output

        self.code_blocks['outputs'] = outputs



#!/usr/bin/env python

import re
import textwrap

from pkg_resources import parse_version
from setuptools import __version__ as SETUPTOOLS_VERSION
import sympy as sm
from sympy.core.function import AppliedUndef
from sympy.utilities.iterables import iterable
from sympy.physics.mechanics import dynamicsymbols

SYMPY_VERSION = sm.__version__


def sympy_equal_to_or_newer_than(version, installed_version=None):
    """Returns true if the installed version of SymPy is equal to or newer
    than the provided version string."""

    if installed_version is None:
        v = SYMPY_VERSION
    else:
        v = installed_version

    if v.endswith('-git') and \
            parse_version(SETUPTOOLS_VERSION) >= parse_version('8.0'):

        msg = ('You are using an older development version of SymPy with a '
               'non-PEP440 compliant version number: {}. Please install '
               'setuptools < 8.0 or a newer development version of SymPy.')
        raise ValueError(msg.format(v))

    return parse_version(v) >= parse_version(version)


def sympy_newer_than(version):
    """Returns true if the installed version of SymPy is newer than the
    provided version string."""
    return parse_version(SYMPY_VERSION) > parse_version(version)


def wrap_and_indent(lines, indentation=4, width=79, continuation=None,
                    comment=None):
    """Returns a single string in which the lines have been indented and
    wrapped into a block of text.

    Parameters
    ==========
    indentation : integer
        The number of characters to indent.
    width : integer
        The maximum line width.
    continuation : string
        The continuation characters.
    comment : string
        The character that designates a comment line.

    """

    if continuation is None:
        cont_len = 0
    else:
        cont_len = len(continuation)

    if comment is None:
        comment_len = 0
    else:
        comment_len = len(comment)

    # TODO : This will indent any lines that only contain a new line. Which
    # may not be preferable.
    new_lines = []

    # TODO : The Octave printer has ".*" and "./" as operators and this doesn't
    # deal with that.
    # add whitespace before and after [*/] binary operands between
    # subexpressions and input/output
    pattern = re.compile('(\w\])([*/])(\w)')
    for line in lines:
        if line != '\n':
            line = pattern.sub(lambda m: ' '.join(m.groups()), line)
            wrapped = textwrap.wrap(line,
                                    width=width-indentation-cont_len-comment_len,
                                    break_long_words=False)
            if continuation:
                last = wrapped[-1]
                wrapped = [l + continuation for l in wrapped[:-1]]
                wrapped.append(last)

            if comment:
                for i, l in enumerate(wrapped[1:]):
                    wrapped[i + 1] = comment + ' ' + l
        else:
            wrapped = [line]
        new_lines += wrapped
    spacer = '\n' + ' ' * indentation
    return ' ' * indentation + spacer.join(new_lines)


# This is a copy of the function in SymPy. It doesn't exist in SymPy 0.7.4
# so we keep it here for now.
def find_dynamicsymbols(expression, exclude=None):
    """Find all dynamicsymbols in expression.

    >>> from sympy.physics.mechanics import dynamicsymbols, find_dynamicsymbols
    >>> x, y = dynamicsymbols('x, y')
    >>> expr = x + x.diff()*y
    >>> find_dynamicsymbols(expr)
    set([x(t), y(t), Derivative(x(t), t)])

    If the optional ``exclude`` kwarg is used, only dynamicsymbols
    not in the iterable ``exclude`` are returned.

    >>> find_dynamicsymbols(expr, [x, y])
    set([Derivative(x(t), t)])
    """
    t_set = set([dynamicsymbols._t])
    if exclude:
        if iterable(exclude):
            exclude_set = set(exclude)
        else:
            raise TypeError("exclude kwarg must be iterable")
    else:
        exclude_set = set()
    return set([i for i in expression.atoms(AppliedUndef, sm.Derivative) if
                i.free_symbols == t_set]) - exclude_set


class PyDyDeprecationWarning(DeprecationWarning):
    pass

class PyDyImportWarning(ImportWarning):
    pass

class PyDyFutureWarning(FutureWarning):
    pass

class PyDyUserWarning(UserWarning):
    pass



from pkg_resources import parse_version
from sympy import ordered, __version__

from ...models import multi_mass_spring_damper
from ..octave_code import OctaveMatrixGenerator

SYMPY_VERSION = __version__
del __version__


def test_OctaveMatrixGenerator():

    expected_m_file = """\
function [output_1] = eval_mats(input_1, input_2, input_3)
% function [output_1] = eval_mats(input_1, input_2, input_3)
%
% input_1 : [x0(t), x1(t), x2(t)]
% input_2 : [v0(t), v1(t), v2(t)]
% input_3 : [c0, c1, c2, k0, k1, k2, m0, m1, m2]

    pydy_0 = input_2(1);
    pydy_1 = input_2(2);
    pydy_2 = input_2(3);
    pydy_3 = input_3(8) + input_3(9);
    pydy_4 = 1./(input_3(7) + pydy_3);
    pydy_5 = -input_3(1).*pydy_0 - input_3(4).*input_1(1);
    pydy_6 = input_3(9).*pydy_4;
    pydy_7 = input_3(9) - pydy_3.*pydy_6;
    pydy_8 = 1./(-pydy_3.^2.*pydy_4 + pydy_3);
    pydy_9 = -input_3(2).*pydy_1 - input_3(5).*input_1(2) - ...
    pydy_3.*pydy_4.*pydy_5;
    pydy_10 = (-input_3(3).*pydy_2 - input_3(6).*input_1(3) - ...
    pydy_5.*pydy_6 - pydy_7.*pydy_8.*pydy_9)./(-input_3(9).^2.*pydy_4 + ...
    input_3(9) - pydy_7.^2.*pydy_8);
    pydy_11 = pydy_8.*(-pydy_10.*pydy_7 + pydy_9);

    output_1 = [pydy_0; pydy_1; pydy_2; pydy_4.*(-input_3(9).*pydy_10 - ...
    pydy_11.*pydy_3 + pydy_5); pydy_11; pydy_10];

end
"""

# SymPy > 1.0 outputs different cse results.
    expected_m_file_new = """\
function [output_1] = eval_mats(input_1, input_2, input_3)
% function [output_1] = eval_mats(input_1, input_2, input_3)
%
% input_1 : [x0(t), x1(t), x2(t)]
% input_2 : [v0(t), v1(t), v2(t)]
% input_3 : [c0, c1, c2, k0, k1, k2, m0, m1, m2]

    pydy_0 = input_2(1);
    pydy_1 = input_2(2);
    pydy_2 = input_2(3);
    pydy_3 = 1./(input_3(7) + input_3(8) + input_3(9));
    pydy_4 = -input_3(1).*pydy_0;
    pydy_5 = -input_3(4).*input_1(1);
    pydy_6 = input_3(8) + input_3(9);
    pydy_7 = -input_3(9).*pydy_3.*pydy_6 + input_3(9);
    pydy_8 = 1./(input_3(8) + input_3(9) - pydy_3.*pydy_6.^2);
    pydy_9 = 1./(-input_3(9).^2.*pydy_3 + input_3(9) - pydy_7.^2.*pydy_8);
    pydy_10 = pydy_4 + pydy_5;
    pydy_11 = -input_3(2).*pydy_1;
    pydy_12 = -input_3(5).*input_1(2);
    pydy_13 = -pydy_10.*pydy_3.*pydy_6;
    pydy_14 = -input_3(3).*pydy_2 - input_3(6).*input_1(3) - ...
    input_3(9).*pydy_10.*pydy_3 - pydy_7.*pydy_8.*(pydy_11 + pydy_12 + ...
    pydy_13);
    pydy_15 = pydy_11 + pydy_12 + pydy_13 - pydy_14.*pydy_7.*pydy_9;

    output_1 = [pydy_0; pydy_1; pydy_2; ...
    pydy_3.*(-input_3(9).*pydy_14.*pydy_9 - pydy_15.*pydy_6.*pydy_8 + ...
    pydy_4 + pydy_5); pydy_15.*pydy_8; pydy_14.*pydy_9];

end
"""

    expected_m_file_1p1 = """\
function [output_1] = eval_mats(input_1, input_2, input_3)
% function [output_1] = eval_mats(input_1, input_2, input_3)
%
% input_1 : [x0(t), x1(t), x2(t)]
% input_2 : [v0(t), v1(t), v2(t)]
% input_3 : [c0, c1, c2, k0, k1, k2, m0, m1, m2]

    pydy_0 = input_2(1);
    pydy_1 = input_2(2);
    pydy_2 = input_2(3);
    pydy_3 = input_3(8) + input_3(9);
    pydy_4 = 1./(input_3(7) + pydy_3);
    pydy_5 = -input_3(1).*pydy_0 - input_3(4).*input_1(1);
    pydy_6 = input_3(9).*pydy_4;
    pydy_7 = input_3(9) - pydy_3.*pydy_6;
    pydy_8 = 1./(-pydy_3.^2.*pydy_4 + pydy_3);
    pydy_9 = -input_3(2).*pydy_1 - input_3(5).*input_1(2) - ...
    pydy_3.*pydy_4.*pydy_5;
    pydy_10 = (-input_3(3).*pydy_2 - input_3(6).*input_1(3) - ...
    pydy_5.*pydy_6 - pydy_7.*pydy_8.*pydy_9)./(-input_3(9).^2.*pydy_4 + ...
    input_3(9) - pydy_7.^2.*pydy_8);
    pydy_11 = pydy_8.*(-pydy_10.*pydy_7 + pydy_9);

    output_1 = [pydy_0; pydy_1; pydy_2; pydy_4.*(-input_3(9).*pydy_10 - ...
    pydy_11.*pydy_3 + pydy_5); pydy_11; pydy_10];

end
"""

    expected_m_file_1p2 = """\
function [output_1] = eval_mats(input_1, input_2, input_3)
% function [output_1] = eval_mats(input_1, input_2, input_3)
%
% input_1 : [x0(t), x1(t), x2(t)]
% input_2 : [v0(t), v1(t), v2(t)]
% input_3 : [c0, c1, c2, k0, k1, k2, m0, m1, m2]

    pydy_0 = input_2(1);
    pydy_1 = input_2(2);
    pydy_2 = input_2(3);
    pydy_3 = input_3(8) + input_3(9);
    pydy_4 = 1./(input_3(7) + pydy_3);
    pydy_5 = input_3(9).*pydy_4;
    pydy_6 = input_3(9) - pydy_3.*pydy_5;
    pydy_7 = 1./(-pydy_3.^2.*pydy_4 + pydy_3);
    pydy_8 = -input_3(1).*pydy_0 - input_3(4).*input_1(1);
    pydy_9 = -input_3(2).*pydy_1 - input_3(5).*input_1(2) - ...
    pydy_3.*pydy_4.*pydy_8;
    pydy_10 = (-input_3(3).*pydy_2 - input_3(6).*input_1(3) - ...
    pydy_5.*pydy_8 - pydy_6.*pydy_7.*pydy_9)./(-input_3(9).^2.*pydy_4 + ...
    input_3(9) - pydy_6.^2.*pydy_7);
    pydy_11 = pydy_7.*(-pydy_10.*pydy_6 + pydy_9);

    output_1 = [pydy_0; pydy_1; pydy_2; pydy_4.*(-input_3(9).*pydy_10 - ...
    pydy_11.*pydy_3 + pydy_8); pydy_11; pydy_10];

end
"""

    sys = multi_mass_spring_damper(3)
    q = sys.coordinates
    u = sys.speeds
    p = list(ordered(sys.constants_symbols))
    sym_rhs = sys.eom_method.rhs()
    g = OctaveMatrixGenerator([q, u, p], [sym_rhs])
    if parse_version(SYMPY_VERSION) >= parse_version('1.2'):
        assert g.doprint() == expected_m_file_1p2
    elif parse_version(SYMPY_VERSION) >= parse_version('1.1'):
        assert g.doprint() == expected_m_file_1p1
    elif parse_version(SYMPY_VERSION) > parse_version('1.0'):
        assert g.doprint() == expected_m_file_new
    else:
        assert g.doprint() == expected_m_file
